// 
// sussen-agent - Main.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Security.Cryptography;
using System.Text;
using System.Xml;
using Mono.Unix;
using Sussen.Agent.Webservice;
using Sussen.Oval;

namespace Sussen.Agent
{
	public class MainApp
	{
		static bool agentSendResults = false;
	
		static void printUsage ()
		{
			string usage = Defines.PACKAGE + " " + Defines.VERSION + "\n" +
				"Copyright 2004-2006 MMG Security, Inc.\n\n";

			usage += 
				"Usage: " + Defines.PACKAGE + " [OPTIONS] <definition file>\n\n" +
				"Options:\n" +
				"  -a,  --analyze [file]\t\tAnalyze previously collected data [file]\n" +
				"  -c,  --collect\t\tCollect data from system only\n" +
				"  -d,  --debug\t\t\tEnable debugging mode\n" +
				"  -f,  --family <name>\t\tSetup the agent for OS <name> (default: linux)\n" +
				"  -h,  --help\t\t\tPrint this usage message.\n" +
				"  -l,  --log <file>\t\tLog program actions to <file>\n" +
				"  -o,  --output <file>\t\tSave results to <file> (def: results.html)\n" +
				"  -ot, --output-type <type>\tOutput type [html|xml] (def: html)\n" +
				"  -r,  --register-agent\t\tRegister agent with web service\n" +
				"  -s,  --scan\t\t\tScans the system for vulnerabilities\n" +
				"  -sr, --send-results\t\tSend results of assessment to web service\n" +
				"  -v,  --version\t\tPrint version and exit.\n" +
				"  -vm, --verify-md5 <md5>\tVerify OVAL defintions file md5 signature.\n";
				
			Console.WriteLine (usage);
		}

		static void printVersion ()
		{
			string version = Defines.PACKAGE + " " + Defines.VERSION + "\n" +
				"Copyright (c) 2004-2006 MMG Security, Inc.\n" +
				"This is free software; see the source for copying conditions. There is NO\n" +
				"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n";

			Console.WriteLine (version);
		}

		// Taken from Banshee; written by Aaron Bockover (aaron@aaronbock.net)
		[DllImport("libc")]
		static extern int prctl(int option, byte [] arg2, ulong arg3 , ulong arg4, ulong arg5);
        
        	static void SetProcessName (string name)
		{
	            if(prctl(15 /* PR_SET_NAME */, Encoding.ASCII.GetBytes(name), 0, 0, 0) != 0) {
        	        throw new ApplicationException("Error setting process name: " + 
                	    Mono.Unix.Native.Stdlib.GetLastError());
	            }
        	}

		static void RegisterAgent (string serviceUri)
		{
			if (serviceUri == null)
				throw new ArgumentNullException ("web service URI");

			if (File.Exists ("sussen-agent.id")) {
				Console.Write ("The agent appears to already be registered.\nWould you like to continue (Yes/No)? ");
				string answer = Console.ReadLine ();
				answer = answer.ToLower ();

				if (answer == "n" || answer == "no") {
					Console.WriteLine ("Aborting.");
					return;
				}
			}

			string hostName = Dns.GetHostName();
			
			try {
				IPHostEntry hostInfo = Dns.GetHostByName(hostName);
				hostName = hostInfo.HostName;
			} catch {}
			
			try  {
			
				ServiceConfig sc = new ServiceConfig (serviceUri);			
				Assembly newAssembly = sc.GetAssembly ();
						
				Type t = newAssembly.GetType ("Sussen.Agent.WebService.SussenService");
				object obj = Activator.CreateInstance (t);
						
				ArrayList methodParams = new ArrayList();
				methodParams.Add (hostName);
				
				MethodInfo regmi = obj.GetType().GetMethod ("registerAgent");			
				object result = regmi.Invoke (obj, (object[])methodParams.ToArray(typeof(object)));

				string agentID = result.ToString();

				Assembly asm = Assembly.GetExecutingAssembly ();
				string asm_dir = System.IO.Path.GetDirectoryName (asm.Location);
				string agentIDFile = Path.Combine (asm_dir, "sussen-agent.id");

				StreamWriter sw = new StreamWriter(agentIDFile);
				sw.WriteLine (agentID);
				sw.Close ();

				Console.WriteLine ("{0} has successfully registered with the following web service:\n{1}.", Defines.PACKAGE, serviceUri);
				
			} catch (Exception e) {
			
				Console.WriteLine ("ERROR registering agent: {0}", e.Message);
			}
		}

		static string GetAgentID ()
		{
			Assembly asm = Assembly.GetExecutingAssembly ();
			string asm_dir = System.IO.Path.GetDirectoryName (asm.Location);
			string agentIDFile = Path.Combine (asm_dir, "sussen-agent.id");

			if (!File.Exists (agentIDFile)) {
				throw new ApplicationException ("Unable to open sussen-agent.id");
			}

			StreamReader sr = new StreamReader (agentIDFile);
			string id = sr.ReadLine ();
			sr.Close ();

			return id;
		}

		static void SendResults (string resultData)
		{
			if (resultData == null)
				throw new ArgumentNullException ("result data");

			try {
			
				ServiceConfig sc = new ServiceConfig (ConfigurationSettings.AppSettings.Get ("webserviceWSDL"));			
				Assembly newAssembly = sc.GetAssembly ();
			
				Type t = newAssembly.GetType ("Sussen.Agent.WebService.SussenService");
				object obj = Activator.CreateInstance (t);

				byte[] encData_byte = new byte [resultData.Length];
				encData_byte = System.Text.Encoding.UTF8.GetBytes (resultData);
				string encodedData = Convert.ToBase64String(encData_byte);

				ArrayList methodParams = new ArrayList();
				methodParams.Add (GetAgentID());
				methodParams.Add (encodedData);
				
				MethodInfo mi = obj.GetType().GetMethod ("sendResults");
				object result = mi.Invoke (obj, (object[])methodParams.ToArray(typeof(object)));
	
				string msg = result.ToString ();
				if (msg.ToLower() != "results saved.")
					throw new Exception ("Unable to save results via web service");
	
			} catch (Exception e) {
			
				Console.WriteLine ("ERROR sending results: {0}", e.Message);
			}
		}

		static ResultOutputTypes GetFileFormat (string fileFormat)
		{
			if (fileFormat == null)
				throw new ArgumentNullException ("output type");
			
			switch (fileFormat.ToLower()) {
			
			case "html":
				return ResultOutputTypes.HTML; 
				
			case "xml":
				return ResultOutputTypes.XML;
				
			default:
				throw new ArgumentOutOfRangeException ("Invalid output type");
			}
		}

		static Config GetConfig (string[] args)
		{
			string definitionFile = null;
			string dataFileName = null;
			string logFileName = null;
			string resultsFileName = null;
			string resultsFileFormat = null;
			string fileHash = null;
			string familyName = null;

			bool debugMode = false;
			bool doScan = false;
			bool collectOnly = false;
			bool analyzeOnly = false;
			bool verifyMD5 = false;

			// Parse command-line arguments 			
			for (int i = 0; i < args.Length; i++) {

				string arg = args[i];

				switch (arg) {
				case "-a":
				case "--analyze":
					++i;					
					dataFileName = i < args.Length ? args[i] : null;				
					analyzeOnly = true;
					break;

				case "-c":
				case "--collect":
					collectOnly = true;
					break;

				case "-d":
				case "--debug":
					debugMode = true;
					break;

				case "-f":
				case "--family":
					++i;					
					familyName = i < args.Length ? args[i] : null;
					break;
					
				case "-h":
				case "--help":
					printUsage ();
					Environment.Exit (0);
					break;

				case "-l":
				case "--log":
					++i;
					logFileName = i < args.Length ? args[i] : null;
					break;

				case "-o":
				case "--output":
					++i;
					resultsFileName = i < args.Length ? args[i] : null;
					break;
					
				case "-ot":
				case "--output-type":
					++i;
					resultsFileFormat = i < args.Length ? args[i] : null;
					break;
					
				case "-r":
				case "--register-agent":
					RegisterAgent (ConfigurationSettings.AppSettings.Get ("webserviceWSDL"));
					Environment.Exit (0);
					break;					

				case "-s":
				case "--scan":
					doScan = true;
					break;

				case "-sr":
				case "--send-results":
					agentSendResults = true;
					break;

				case "-v":
				case "--version":
					printVersion ();
					Environment.Exit (0);
					break;

				case "-vm":
				case "--verify-md5":
					++i;
					fileHash = i < args.Length ? args[i] : null;
					verifyMD5 = true;
					break;
					
				default:
					definitionFile = args[i];
					break;
				}
			}

			if (familyName == null)
				familyName = "linux";

			Config config = new Config (familyName);
			config.DefinitionsFile = definitionFile;
			config.XmlDirectory = ConfigurationSettings.AppSettings.Get ("sussenXmlDirectory");
			config.DownloadDirectory = ConfigurationSettings.AppSettings.Get ("sussenDownloadDirectory");
			config.ResultsFile = "results.html";
			config.ResultsFileFormat = ResultOutputTypes.HTML;

			if (dataFileName != null)
				config.SystemCharacteristicsFile = dataFileName;

			if (analyzeOnly) {			
				config.AnalyzeOnly = true;
				config.CollectOnly = false;
			}
			
			if (collectOnly) {
				config.AnalyzeOnly = false;
				config.CollectOnly = true;			
			}

			if (!doScan && !analyzeOnly && !collectOnly) {
				Console.WriteLine ("No operation specified. Exiting");
				Environment.Exit (0);
			}

			if (verifyMD5) {
				config.DefinitionsFileHash = fileHash;
				config.VerifyMD5 = true;
			}

			if (logFileName != null) 
				config.LogFile = logFileName;

			if (debugMode) {
				config.LogFile = "-";
				config.LoggingLevel = LogLevel.Debug;
			}

			if (resultsFileName != null)
				config.ResultsFile = resultsFileName;

			if (resultsFileFormat != null)
				config.ResultsFileFormat = GetFileFormat (resultsFileFormat);
			
			return config;
		}

		static void Main (string[] args) 
		{
			try {
				SetProcessName ("sussen-agent");
			} catch {}

			if (args.Length == 0) {
				printUsage ();
				Environment.Exit (0);
			} 

			Config config = null;

			try {
			
				config = GetConfig (args);			
				
			} catch (Exception e) {
			
				Console.WriteLine (e.Message);
				Environment.Exit (0);
				
			}

			if (!File.Exists (config.DefinitionsFile)) {
				Console.WriteLine ("ERROR: OVAL definitions file '{0}' does not exist.", config.DefinitionsFile);
				Environment.Exit (0);
			}

			if (config.AnalyzeOnly && !File.Exists (config.SystemCharacteristicsFile)) {
				Console.WriteLine ("ERROR: Data file '{0}' does not exist.", config.SystemCharacteristicsFile);
				Environment.Exit (0);			
			}

			try {

				Interpreter i = new Interpreter (config);
				i.CollectData ();
				
				if (!config.CollectOnly)
					i.AnalyzeData ();
			
				if (agentSendResults)
					SendResults (i.Report);
			
			} catch (InterpreterException ie) {			
				Console.WriteLine (ie.Message);
				Environment.Exit (0);			
			} catch (Exception e) {
				Console.WriteLine ("Unknown error: {0}", e.ToString());
				Environment.Exit (0);
			}
		}
	}
}
