// 
// sussen-agent - Webservice.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Net;
using System.Reflection;
using System.Text;
using System.Web.Services.Description;
using System.Web.Services.Discovery;
using System.Xml.Schema;
using Microsoft.CSharp;

namespace Sussen.Agent.Webservice
{
	public class AssemblyGenerator
	{
		public AssemblyGenerator ()
		{
		}
		
		public Assembly Build (ArrayList descriptions, ArrayList schemas)
		{
			CodeNamespace codeNamespace = new CodeNamespace ("Sussen.Agent.WebService");

			ServiceDescriptionImporter importer = new ServiceDescriptionImporter();
			importer.ProtocolName = "Soap";
			
			foreach (ServiceDescription sd in descriptions)
				importer.AddServiceDescription(sd, null, null);
					
			foreach (XmlSchema sc in schemas)
				importer.Schemas.Add (sc);
				
			importer.Import(codeNamespace, null);

			CodeDomProvider provider = new CSharpCodeProvider();
			ICodeGenerator generator = provider.CreateGenerator();

			StringBuilder sb = new StringBuilder();
			StringWriter sw = new StringWriter (sb);
				
			generator.GenerateCodeFromNamespace (codeNamespace, sw, null);
			string src = sb.ToString();
			sw.Close();
			
			CompilerParameters cp = new CompilerParameters();
			cp.ReferencedAssemblies.Add("System.dll");
			cp.ReferencedAssemblies.Add("System.Xml.dll");
			cp.ReferencedAssemblies.Add("System.Web.Services.dll");
			cp.ReferencedAssemblies.Add(System.Reflection.Assembly.GetExecutingAssembly().Location);
			
			ICodeCompiler compiler = provider.CreateCompiler();
			CompilerResults cr = compiler.CompileAssemblyFromSource (cp, src);
				
			if (cr.Errors.Count > 0) {
				foreach (CompilerError ce in cr.Errors)
                    Console.WriteLine(ce.ToString());
                    
				throw new Exception(string.Format("Build failed: {0} errors", cr.Errors.Count));
			}

			return cr.CompiledAssembly; 				
		}
	}

	public class ServiceConfig
	{
		AssemblyGenerator generator;

		ArrayList descriptions;
		ArrayList schemas;

		string wsdlLocation;

		public ServiceConfig (string wsdlUri)
		{
			wsdlLocation = wsdlUri;
			
			generator = new AssemblyGenerator ();

			descriptions = new ArrayList ();
			schemas = new ArrayList ();
		}
		
		public Assembly GetAssembly ()
		{
			DiscoveryClientProtocol dcc = new DiscoveryClientProtocol ();

			string domain = ConfigurationSettings.AppSettings.Get ("webserviceDomain");
			string user = ConfigurationSettings.AppSettings.Get ("webserviceUser");
			string pass = ConfigurationSettings.AppSettings.Get ("webservicePass");
			
			if (domain != "" || domain != null || user != "" || user != null || pass != "" || pass != null) {
				
				NetworkCredential credentials = new NetworkCredential();
				
				if (user != "" || user != null)
					credentials.UserName = user;
					
				if (pass != "" || pass != null)
					credentials.Password = pass;
					
				if (domain != "" || domain != null)
					credentials.Domain = domain;
					
				dcc.Credentials = credentials;
			}

			dcc.AllowAutoRedirect = true;			
			dcc.DiscoverAny (wsdlLocation);
			dcc.ResolveAll ();

			foreach (object doc in dcc.Documents.Values) {
			
				if (doc is ServiceDescription)
					descriptions.Add ((ServiceDescription)doc);
				else if (doc is XmlSchema)
					schemas.Add ((XmlSchema)doc);
			}
			
			if (descriptions.Count == 0)
				throw new Exception ("No WSDL document was found at " + wsdlLocation);
						
			Assembly asm = generator.Build (descriptions, schemas);
			
			return asm;
		}
	}
}
