// 
// sussen-applet - Applet.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Runtime.InteropServices;
using System.Text;
using Gtk;
using Gnome;
using Mono.Unix;

namespace Sussen.Applet
{
	public class SussenPanelApplet : Gnome.PanelApplet
	{	
		static Gnome.Program program;
		static System.Object dbus_connection;
		static Gdk.Pixbuf panelIcon;
		static Gdk.Pixbuf sussenLogo;
	
		protected SussenPanelApplet (IntPtr raw) : base (raw)
		{
			panelIcon = new Gdk.Pixbuf (null, "sussen-icon.png"); 
			sussenLogo = new Gdk.Pixbuf (null, "sussen.png");
		}

		static void printUsage ()
		{
			string usage = Defines.PACKAGE + "-applet " + Defines.VERSION + "\n" +
				"Copyright 2004-2006 MMG Security, Inc.\n\n";

			usage += 
				"Usage: " + Defines.PACKAGE + "-applet [OPTIONS]\n\n" +
				"Options:\n" +
				"  --tray-icon\t\tRun as system tray icon instead of GNOME applet.\n" +
				"  --version\t\tPrint version and exit.\n" +
				"  --help\t\tPrint this usage message.\n";

			Console.WriteLine (usage);
		}

		static void printVersion ()
		{
			string version = Defines.PACKAGE + " " + Defines.VERSION + "\n" +
				"Copyright (c) 2004-2006 MMG Security, Inc.\n" +
				"This is free software; see the source for copying conditions. There is NO\n" +
				"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n";

			Console.WriteLine (version);
		}

		// Taken from Banshee; written by Aaron Bockover (aaron@aaronbock.net)
        [DllImport("libc")]
        static extern int prctl(int option, byte [] arg2, ulong arg3 , ulong arg4, ulong arg5);
        
        	static void SetProcessName (string name)
	    {
	            if(prctl(15 /* PR_SET_NAME */, Encoding.ASCII.GetBytes(name), 0, 0, 0) != 0) {
        	        throw new ApplicationException("Error setting process name: " + 
                	    Mono.Unix.Native.Stdlib.GetLastError());
	            }
        	}

		static void Main (string[] args)
		{
			bool runAsTrayIcon = false;
		
			foreach (string arg in args) {
			
				switch (arg) {
				
				case "--tray-icon":
					runAsTrayIcon = true;
					break;
				
				case "--help":
					printUsage ();
					Environment.Exit (0);
					break;				
				
				case "--version":
					printVersion ();
					Environment.Exit (0);
					break;
				}
			}
		
			try {
				SetProcessName ("sussen-applet");
			} catch {}

			RegisterRemoteControl ();
		
			program = new Gnome.Program (Defines.PACKAGE, Defines.VERSION, Gnome.Modules.UI, args);
						
			if (runAsTrayIcon)
				StartTrayIcon ();
			else
				PanelAppletFactory.Register (typeof (SussenPanelApplet));
		}

		static void RegisterRemoteControl ()
		{
#if ENABLE_DBUS
			try {
			
				dbus_connection = DBus.Bus.GetSessionBus ();
				DBus.Service service = new DBus.Service ((DBus.Connection) dbus_connection, RemoteControlProxy.Namespace);

				RemoteControl remote_control = new RemoteControl ();
				service.RegisterObject (remote_control, RemoteControlProxy.Path);

				Console.WriteLine ("dbus remote control registered.");
				
			} catch (Exception e) {
				Console.WriteLine (e.ToString());
			}
#endif
		}
		
		static void StartTrayIcon ()
		{
			SussenTrayIcon trayIcon = new SussenTrayIcon ();
			trayIcon.Show ();
			
			Console.WriteLine ("If you're wondering whether {0} is working check your system tray", Defines.PACKAGE);
			
			program.Run ();
		}
		
		public override string IID 
		{
			get { return "OAFIID:SussenApplet"; }
		}

		public override string FactoryIID 
		{
			get { return "OAFIID:SussenApplet_Factory"; }
		}

		public override void Creation ()
		{								
			Gtk.Image image = new Gtk.Image (panelIcon);
			this.Add (image);			
			this.ShowAll ();	
			
			BonoboUIVerb [] menuVerbs;
			menuVerbs = new BonoboUIVerb [] {
				new BonoboUIVerb ("Scan", new ContextMenuItemCallback (ShowScanSystemDialog)),
				new BonoboUIVerb ("Analyze", new ContextMenuItemCallback (ShowAnalyzeDataDialog)),
				new BonoboUIVerb ("About", new ContextMenuItemCallback (ShowAbout))
			};
			
			this.SetupMenu (Defines.MENU_XML, menuVerbs);

			Gtk.Tooltips tips = new Gtk.Tooltips ();
			tips.SetTip (this, "Security scanner", null);
			tips.Enable ();
			tips.Sink ();
			
			Console.WriteLine ("{0} created.", Defines.PACKAGE);
			
			program.Run ();
		}

		void ShowScanSystemDialog ()
		{
			new ActionDialog (ActionDialogType.Scan);
		}

		void ShowAnalyzeDataDialog ()
		{
			new ActionDialog (ActionDialogType.Analyze);
		}
		
		void ShowAbout ()
		{
			string[] author = { "Loren Bandiera" };
			string[] docs = { "Loren Bandiera" };
			string translators = "";
			string desc = "Security scanner";
			string copy = "Copyright \xa9 2005 MMG Security, Inc.";
			
			Gnome.About ab = new Gnome.About (
				Defines.PACKAGE,
				Defines.VERSION,
				copy,
				desc,
				author,
				docs, 
				translators, 
				sussenLogo);

			ab.Icon = panelIcon;
			ab.Show ();
		}
	}
}
