// 
// sussen-editor - Main.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Runtime.InteropServices;
using System.Text;
using Mono.Unix;
using Gtk;
using Gnome;

namespace Sussen.Editor
{
	public class MainApp
	{
		static void printUsage ()
		{
			string usage = "sussen-editor " + Defines.VERSION + "\n" +
				"Copyright (c) 2005 MMG Security, Inc.\n\n";

			usage += 
				"Usage: sussen-editor [OPTIONS]\n\n" +
				"Options:\n" +
				"  -d,  --debug\t\t\tDebug mode.\n" +
				"  -h,  --help\t\t\tPrint this usage message.\n" +
				"  -v,  --version\t\tPrint version and exit.\n";
				
			Console.WriteLine (usage);
		}

		static void printVersion ()
		{
			string version = Defines.PACKAGE + " " + Defines.VERSION + "\n" +
				"Copyright (c) 2005 MMG Security, Inc.\n" +
				"This is free software; see the source for copying conditions. There is NO\n" +
				"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n";

			Console.WriteLine (version);
		}

		// Taken from Banshee; written by Aaron Bockover (aaron@aaronbock.net)
        [DllImport("libc")]
        static extern int prctl(int option, byte [] arg2, ulong arg3 , ulong arg4, ulong arg5);
        
        	static void SetProcessName (string name)
	    {
	            if(prctl(15 /* PR_SET_NAME */, Encoding.ASCII.GetBytes(name), 0, 0, 0) != 0) {
        	        throw new ApplicationException("Error setting process name: " + 
                	    Mono.Unix.Native.Stdlib.GetLastError());
	            }
        	}

		static void Main (string[] args) 
		{
			bool debugMode = false;
		
			// Parse command-line arguments 			
			for (int i = 0; i < args.Length; i++) {

				string arg = args[i];

				switch (arg) {

				case "-d":
				case "--debug":
					debugMode = true;
					break;
				
				case "-h":
				case "--help":
					printUsage ();
					Environment.Exit (0);
					break;

				case "-v":
				case "--version":
					printVersion ();
					Environment.Exit (0);
					break;
				}
			}

			try {
				SetProcessName ("sussen-editor");
			} catch {}

			Application.Init ();

			Gnome.Program program = new Gnome.Program (Defines.PACKAGE, Defines.VERSION, Modules.UI, args);

			new MainWindow (debugMode);
			
			program.Run ();
		}
	}
}
