// 
// sussen-editor - Window.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 .
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Text;
using System.Xml;

using Gtk;
using Gnome;
using GLib;

using Sussen.Oval;

namespace Sussen.Editor
{
	public struct DefinitionMetaData
	{
		public string ID;
		public string Title;
		public string Family;
		public string Platform;
		public string Product;
		public string RefSource;
		public string RefID;
		public string RefUrl;
		public string Description;
		
		public string CriteriaOperator;
		public string[] TestCriteria;
	};

	public class MainWindow
	{
		Glade.XML ui;
		
		[Glade.Widget] Gtk.Window mainWindow;
		[Glade.Widget] Gtk.HPaned hpaned1;
		[Glade.Widget] Gtk.TreeView fileTreeView;
		[Glade.Widget] Gtk.Notebook noteBook;
		
		// Definition
		[Glade.Widget] Gtk.Entry defIDEntry;
		[Glade.Widget] Gtk.Entry defTitleEntry;
		[Glade.Widget] Gtk.ComboBox defFamilyComboBox;
		[Glade.Widget] Gtk.Entry defPlatformEntry;
		[Glade.Widget] Gtk.Entry defProductEntry;
		[Glade.Widget] Gtk.Entry defRefSourceEntry;
		[Glade.Widget] Gtk.Entry defRefIDEntry;
		[Glade.Widget] Gtk.Entry defRefUrlEntry;
		[Glade.Widget] Gtk.TextView defDescTextView;
		[Glade.Widget] Gtk.ComboBox defCritOperatorComboBox;
		[Glade.Widget] Gtk.TreeView defCriteriaTreeView;					
		
		// Test
		[Glade.Widget] Gtk.Entry testIDEntry;
		[Glade.Widget] Gtk.Entry testCommentEntry;
		[Glade.Widget] Gtk.ComboBox testCheckComboBox;
		[Glade.Widget] Gtk.TreeView testObjectsTreeView;
		[Glade.Widget] Gtk.TreeView testStatesTreeView;
		
		static Gdk.Pixbuf sussenIcon;
		static Gdk.Pixbuf sussenLogo;
		
		TreeStore fileStore;
		TreeIter rootIter;
		TreeIter defIter;
		TreeIter testIter;
		TreeIter currentDefIter;
		TreeIter currentTestIter;
		
		ListStore defTestStore;		
		ListStore testObjectsStore;
		ListStore testStatesStore;
		
		Hashtable currentTestObjects;
		Hashtable currentTestStates;
		
		ArrayList currentTests;
		ArrayList currentDefinitions;
		
		string currentTestType;
		bool debugMode;
		
		public MainWindow (bool debug)
		{
			debugMode = debug;
			
			currentTests = new ArrayList ();
			currentDefinitions = new ArrayList ();
		
			currentTestObjects = new Hashtable();
			currentTestStates = new Hashtable();
		
			ui = Glade.XML.FromAssembly ("sussen-editor.glade", "mainWindow", null);
			ui.Autoconnect (this);
					
			hpaned1.Position = 250;
			
			sussenIcon = new Gdk.Pixbuf (null, "sussen-icon.png"); 
			sussenLogo = new Gdk.Pixbuf (null, "sussen.png");
				
			mainWindow.Icon = sussenIcon;			
			mainWindow.Title = "untitled - sussen-editor";
			
			noteBook.ShowTabs = false;
			noteBook.Page = 2;
			
			defTestStore = new ListStore (typeof (string), typeof (string));
			defCriteriaTreeView.Model = defTestStore;
			defCriteriaTreeView.AppendColumn ("ID", new CellRendererText (), "text", 0);
			defCriteriaTreeView.AppendColumn ("Comment", new CellRendererText (), "text", 1);
			
			testObjectsStore = new ListStore (typeof (string), typeof (string));
			testObjectsTreeView.Model = testObjectsStore;
			testObjectsTreeView.AppendColumn ("Name", new CellRendererText (), "text", 0);
			
			CellRendererText cell = new CellRendererText ();
			cell.Editable = true;
			cell.Edited += new EditedHandler (OnObjectEdit);			
			
			testObjectsTreeView.AppendColumn ("Value", cell, "text", 1);
			
			testStatesStore = new ListStore (typeof (string), typeof (string));
			testStatesTreeView.Model = testStatesStore;
			testStatesTreeView.AppendColumn ("Name", new CellRendererText (), "text", 0);
			
			cell = new CellRendererText ();
			cell.Editable = true;
			cell.Edited += new EditedHandler (OnStateEdit);
			
			testStatesTreeView.AppendColumn ("Value", cell, "text", 1);
			
			fileStore = new TreeStore (typeof (string));
			fileTreeView.Model = fileStore;
			fileTreeView.ButtonPressEvent += new ButtonPressEventHandler (OnFileRightClick);
			fileTreeView.RowActivated += new RowActivatedHandler (OnRowActivated);
			
			fileTreeView.Sensitive = false;
		}

		void DoPopUp()
		{
			Menu popup = new Menu();

			ImageMenuItem newItem = new ImageMenuItem ("New definition");
			Gtk.Image newImage = new Gtk.Image (Gtk.Stock.New, IconSize.Menu);
			newItem.Image = newImage;
			newItem.Activated += new EventHandler (OnNewDefinitionActivate);
			newItem.Show ();

			popup.Append (newItem);
			
			ImageMenuItem newItem2 = new ImageMenuItem ("New test");
			Gtk.Image newImage2 = new Gtk.Image (Gtk.Stock.New, IconSize.Menu);
			newItem2.Image = newImage2;
			newItem2.Show ();

			Gtk.Menu typeMenu = new Gtk.Menu ();

//			MenuItem newDpkgInfoTestItem = new MenuItem ("DPKG info test");
//			newDpkgInfoTestItem.Activated += new EventHandler (OnNewTestActivate);
//			newDpkgInfoTestItem.Show ();
//			typeMenu.Append (newDpkgInfoTestItem);
			
			MenuItem newFileTestItem = new MenuItem ("File test");
			newFileTestItem.Activated += new EventHandler (OnNewFileTest);
			newFileTestItem.Show ();
			typeMenu.Append (newFileTestItem);

			MenuItem newInetListeningTest = new MenuItem ("Inet Listening Servers test");
			newInetListeningTest.Activated += new EventHandler (OnNewInetListeningServersTest);
			newInetListeningTest.Show ();
			typeMenu.Append (newInetListeningTest);

//			MenuItem newInetdTest = new MenuItem ("Inetd test");
//			newInetdTest.Activated += new EventHandler (OnNewTestActivate);
//			newInetdTest.Show ();
//			typeMenu.Append (newInetdTest);
			
			MenuItem newInterfaceItem = new MenuItem ("Interface test");
			newInterfaceItem.Activated += new EventHandler (OnNewInterfaceTest);
			newInterfaceItem.Show ();
			typeMenu.Append (newInterfaceItem);
			
			MenuItem newPasswordItem = new MenuItem ("Password test");
			newPasswordItem.Activated += new EventHandler (OnNewPasswordTest);
			newPasswordItem.Show ();
			typeMenu.Append (newPasswordItem);				

			MenuItem newProcessItem = new MenuItem ("Process test");
			newProcessItem.Activated += new EventHandler (OnNewProcessTest);
			newProcessItem.Show ();
			typeMenu.Append (newProcessItem);
			
			MenuItem newRpmItem = new MenuItem ("RPM info test");
			newRpmItem.Activated += new EventHandler (OnNewRpmInfoTest);
			newRpmItem.Show ();
			typeMenu.Append (newRpmItem);				

			MenuItem newRunLevelItem = new MenuItem ("Run level test");
			newRunLevelItem.Activated += new EventHandler (OnNewRunLevelTest);
			newRunLevelItem.Show ();
			typeMenu.Append (newRunLevelItem);			

//			MenuItem newSccsItem = new MenuItem ("SCCS test");
//			newSccsItem.Activated += new EventHandler (OnNewTestActivate);
//			newSccsItem.Show ();
//			typeMenu.Append (newSccsItem);

			MenuItem newShadowItem = new MenuItem ("Shadow test");
			newShadowItem.Activated += new EventHandler (OnNewShadowTest);
			newShadowItem.Show ();
			typeMenu.Append (newShadowItem);			

			MenuItem newUnameItem = new MenuItem ("Uname test");
			newUnameItem.Activated += new EventHandler (OnNewUnameTest);
			newUnameItem.Show ();
			typeMenu.Append (newUnameItem);
			
			newItem2.Submenu = typeMenu;
			popup.Append (newItem2);
			
			popup.Popup(null, null, null, 3, Gtk.Global.CurrentEventTime);
		}

		string GetFamilySelection ()
		{
			TreeIter iter;
				
			if (!defFamilyComboBox.GetActiveIter (out iter))
				return "";

			string family = (string) defFamilyComboBox.Model.GetValue (iter, 0);
			
			return family;
		}

		void SetFamilySelection (string familyName)
		{
			switch (familyName.ToLower()) {
			
			case "linux":
				defFamilyComboBox.Active = 0;
				break;

			case "unix":
				defFamilyComboBox.Active = 1;
				break;				

			default:
				defFamilyComboBox.Active = 0;
				break;
			}
		}
		
		void SetCriteriaOperator (string operatorName)
		{
			switch (operatorName.ToLower()) {
			
			case "and":
				defCritOperatorComboBox.Active = 0;
				break;

			case "or":
				defCritOperatorComboBox.Active = 1;
				break;				

			case "xor":
				defCritOperatorComboBox.Active = 2;
				break;

			default:
				defCritOperatorComboBox.Active = 0;
				break;
			}		
		}

		void SetTestCheck (string checkName)
		{
			switch (checkName.ToLower()) {
			
			case "all":
				testCheckComboBox.Active = 0;
				break;

			case "at least one":
				testCheckComboBox.Active = 1;
				break;				

			case "only one":
				testCheckComboBox.Active = 2;
				break;

			case "none":
				testCheckComboBox.Active = 3;
				break;

			default:
				testCheckComboBox.Active = 0;
				break;
			}		
		}

		#region event_handlers		

		void OnRowActivated (object o, RowActivatedArgs args)
		{	
			TreePath path = args.Path;
			TreeIter iter;
			
			if (fileStore.GetIter (out iter, path)) {

				string name = null;
				name = (string) fileStore.GetValue (iter, 0);

				if (fileStore.IsAncestor (defIter, iter)) {
					foreach (DefinitionMetaData dmd in currentDefinitions) {
						if (dmd.ID == name) {
							SetCurrentDefintionData (dmd);
						}
					}				
					noteBook.Page = 0;
					
				} else if (fileStore.IsAncestor (testIter, iter)) {				
					foreach (TestMetaData tmd in currentTests) {
						if (tmd.ID == name) {
							SetCurrentTestData (tmd);
						}
					}
					
					noteBook.Page = 1;
				}				
			} 		
		}

		public void OnNewActivate (object o, EventArgs args)
		{	
			currentTestObjects.Clear ();
			currentTestStates.Clear ();
			
			fileTreeView.AppendColumn ("fileRoot", new CellRendererText (), "text", 0);
			rootIter = fileStore.AppendValues ("Untitled");
			
			defIter = fileStore.AppendValues (rootIter, "Definitions");
			testIter = fileStore.AppendValues (rootIter, "Tests");			
			fileTreeView.ExpandAll ();
			
			fileTreeView.Sensitive = true;
		}

		void OnObjectEdit (object o, EditedArgs args)
		{
			TreeIter iter;

			if (!testObjectsStore.GetIterFromString (out iter, args.Path))
				return;

			string name = (string) testObjectsStore.GetValue (iter, 0);
			string oldText = (string) testObjectsStore.GetValue (iter, 1);

			if (oldText.Equals (args.NewText))
				return;
			
			testObjectsStore.SetValue (iter, 1, args.NewText);
			currentTestObjects[name] = args.NewText;
		}

		void OnStateEdit (object o, EditedArgs args)
		{
			TreeIter iter;

			if (!testStatesStore.GetIterFromString (out iter, args.Path))
				return;

			string name = (string) testStatesStore.GetValue (iter, 0);
			string oldText = (string) testStatesStore.GetValue (iter, 1);

			if (oldText.Equals (args.NewText))
				return;
			
			testStatesStore.SetValue (iter, 1, args.NewText);
			currentTestStates[name] = args.NewText;
		}

		void GetCurrentTestData ()
		{
			TreeIter iter;
				
			if (!testCheckComboBox.GetActiveIter (out iter))
				return;

			string check = (string) testCheckComboBox.Model.GetValue (iter, 0);
		
			if (testIDEntry.Text == "")
				return;
		
			TestMetaData tmd = new TestMetaData();
			tmd.ID = testIDEntry.Text;
			tmd.Comment = testCommentEntry.Text;				
			tmd.Check = check;
			tmd.TestType = currentTestType;
			tmd.TestObjects = new Hashtable ();
			tmd.TestStates = new Hashtable ();
			
			foreach (string k in currentTestObjects.Keys)
				tmd.TestObjects[k] = currentTestObjects[k];

			foreach (object[] row in testStatesStore) {
				string stateName = (string) row[0];
				string stateValue = (string) row[1];
				
				if (stateValue != "") {
					tmd.TestStates[stateName] = stateValue;
				}
			}
	
			currentTests.Add (tmd);			
			fileStore.SetValue (currentTestIter, 0, tmd.ID);
		}

		void SetCurrentTestData (TestMetaData tmd)
		{	
			testIDEntry.Text = tmd.ID;
			testCommentEntry.Text = tmd.Comment;				
			currentTestType = tmd.TestType;
			
			SetTestCheck (tmd.Check);
			
			foreach (string k in tmd.TestObjects.Keys)
				testObjectsStore.AppendValues (k, (string) tmd.TestObjects[k]); 

			foreach (string k in tmd.TestStates.Keys)
				testStatesStore.AppendValues (k, (string) tmd.TestStates[k]); 
		}

		void GetCurrentDefintionData ()
		{
			TreeIter iter;
					
			if (!defCritOperatorComboBox.GetActiveIter (out iter))
				return;		
		
			string critOperator = (string) defCritOperatorComboBox.Model.GetValue (iter, 0);		
		
			if (defIDEntry.Text == "")
				return;
			
			DefinitionMetaData dmd = new DefinitionMetaData ();
			dmd.ID = defIDEntry.Text;
			dmd.Title = defTitleEntry.Text;
			dmd.Family = GetFamilySelection ();
			dmd.Platform = defPlatformEntry.Text;
			dmd.Product = defProductEntry.Text;
			dmd.RefSource = defRefSourceEntry.Text;
			dmd.RefID = defRefIDEntry.Text;
			dmd.RefUrl = defRefUrlEntry.Text;
			dmd.Description = defDescTextView.Buffer.Text;
	
			dmd.CriteriaOperator = critOperator;
			
			ArrayList tmp = new ArrayList ();
			foreach (object[] row in defTestStore)
				tmp.Add (row[0]);
				
			dmd.TestCriteria = (string[]) tmp.ToArray (typeof(string));

			currentDefinitions.Add (dmd);
			fileStore.SetValue (currentDefIter, 0, dmd.ID);
		}

		void SetCurrentDefintionData (DefinitionMetaData dmd)
		{		
			defIDEntry.Text = dmd.ID;
			defTitleEntry.Text = dmd.Title;			
			defPlatformEntry.Text = dmd.Platform;
			defProductEntry.Text = dmd.Product;
			defRefSourceEntry.Text = dmd.RefSource;
			defRefIDEntry.Text = dmd.RefID;
			defRefUrlEntry.Text = dmd.RefUrl;
			defDescTextView.Buffer.Text = dmd.Description;

			SetFamilySelection (dmd.Family);
			SetCriteriaOperator (dmd.CriteriaOperator);	
	
			foreach (string s in dmd.TestCriteria)
				defTestStore.AppendValues (s, "");
		}

		public void OnApplyClicked (object o, EventArgs args)
		{
			if (noteBook.CurrentPage == 0)
				GetCurrentDefintionData ();
			else if (noteBook.CurrentPage == 1)
				GetCurrentTestData ();
		}

		void OnNewDefinitionActivate (object o, EventArgs args)
		{
			currentDefIter = fileStore.AppendValues (defIter, "untitled definition");
			fileTreeView.ExpandAll ();
			defIDEntry.Text = "untitled definition";
		
			noteBook.Page = 0;
		}

		void UpdateTestValues (string[] objs, string[] states)
		{
			testObjectsStore.Clear ();
			
			if (objs != null) 
				foreach (string s in objs)
					testObjectsStore.AppendValues (s, "");
		
			testStatesStore.Clear ();
			foreach (string s in states)
				testStatesStore.AppendValues (s, "");		
		}

		public void OnNewFileTest (object o, EventArgs args)
		{
			string[] objs = { "path", "filename" };
			string[] states = { 
				"type", 
				"group_id",
				"user_id",
				"a_time",
				"c_time",
				"m_time",
				"size",
				"suid",
				"sgid",
				"sticky",
				"uread",
				"uwrite",
				"uexec",
				"gread",
				"gwrite",
				"gexec",
				"oread",
				"owrite",
				"oexec"
			};

			currentTestType = "file_test";
			UpdateTestValues (objs, states);				
			OnNewTestActivate (o, args);
		}

		public void OnNewInetListeningServersTest (object o, EventArgs args)
		{
			string[] objs = { "program_name" };
			string[] states = { 
				"local_address", 
				"local_full_address",
				"local_port",
				"foreign_address",
				"foreign_full_address",
				"foreign_port",
				"pid",
				"protocol",
				"user_id"
			};
			
			currentTestType = "inetlisteningservers_test";
			UpdateTestValues (objs, states);
			OnNewTestActivate (o, args);
		}

		public void OnNewInterfaceTest (object o, EventArgs args)
		{
			string[] objs = { "name" };
			string[] states = { 
				"hardware_addr", 
				"inet_addr",
				"broadcast_addr",
				"netmask",
				"flag"
			};
			
			currentTestType = "interface_test";
			UpdateTestValues (objs, states);
			OnNewTestActivate (o, args);
		}

		public void OnNewPasswordTest (object o, EventArgs args)
		{
			string[] objs = { "username" };
			string[] states = { 
				"password", 
				"user_id",
				"group_id",
				"gcos",
				"home_dir",
				"login_shell"
			};
			
			currentTestType = "password_test";
			UpdateTestValues (objs, states);
			OnNewTestActivate (o, args);
		}

		public void OnNewProcessTest (object o, EventArgs args)
		{
			string[] objs = { "command" };
			string[] states = { 
				"exec_time", 
				"pid",
				"ppid",
				"priority",
				"scheduling_class",
				"start_time",
				"tty",
				"user_id"
			};
			
			currentTestType = "process_test";
			UpdateTestValues (objs, states);
			OnNewTestActivate (o, args);
		}

		public void OnNewRpmInfoTest (object o, EventArgs args)
		{
			string[] objs = { "name" };
			string[] states = { 
				"arch", 
				"epoch",
				"release",
				"version",
				"evr",
				"signature_keyid"
			};
			
			currentTestType = "rpminfo_test";
			UpdateTestValues (objs, states);
			OnNewTestActivate (o, args);
		}

		public void OnNewRunLevelTest (object o, EventArgs args)
		{
			string[] objs = { "service_name", "runlevel" };
			string[] states = { 
				"start", 
				"kill"
			};

			currentTestType = "runlevel_test";
			UpdateTestValues (objs, states);
			OnNewTestActivate (o, args);
		}

		public void OnNewShadowTest (object o, EventArgs args)
		{
			string[] objs = { "username" };
			string[] states = { 
				"password", 
				"chg_lst",
				"chg_allow",
				"chg_req",
				"exp_warn",
				"exp_inact",
				"exp_date",
				"flag"
			};
			
			currentTestType = "shadow_test";
			UpdateTestValues (objs, states);
			OnNewTestActivate (o, args);
		}

		public void OnNewUnameTest (object o, EventArgs args)
		{
			string[] states = { 
				"machine_class", 
				"node_name",
				"os_name",
				"os_release",
				"os_version",
				"processor_type"
			};
			
			currentTestType = "uname_test";
			UpdateTestValues (null, states);
			OnNewTestActivate (o, args);
		}

		public void OnAddTestClicked (object o, EventArgs args)
		{
			AddTestDialog atd = new AddTestDialog (currentTests);			
			if (atd.SelectedTests == null)
				return;
			
			foreach (TestMetaData td in atd.SelectedTests)
				defTestStore.AppendValues (td.ID, td.Comment);
		}
		
		public void OnRemoveTestClicked (object o, EventArgs args)
		{
			TreeModel model;
			TreeIter iter;

			if (!defCriteriaTreeView.Selection.GetSelected (out model, out iter))
				return;
			
			defTestStore.Remove (ref iter);
		}

		void OnNewTestActivate (object o, EventArgs args)
		{
			currentTestIter = fileStore.AppendValues (testIter, "untitled test");
			fileTreeView.ExpandAll ();
			
			testIDEntry.Text = "untitled test";
			testCheckComboBox.Active = 0;
			
			noteBook.Page = 1;
		}

		XmlDocument GenerateDefinitionXml (DefinitionMetaData[] dmd, TestMetaData[] tmd)
		{
			XmlDocument doc = new XmlDocument ();

			XmlNode xmlnode = doc.CreateNode (XmlNodeType.XmlDeclaration,"","");
			doc.AppendChild (xmlnode);
			
			XmlElement oval_definitions = doc.CreateElement ("oval_definitions");
			oval_definitions.SetAttribute ("xmlns", "http://oval.mitre.org/XMLSchema/oval-definitions-5");
			oval_definitions.SetAttribute ("xmlns:oval", "http://oval.mitre.org/XMLSchema/oval-common-5");
			oval_definitions.SetAttribute ("xmlns:oval-def", "http://oval.mitre.org/XMLSchema/oval-definitions-5");
			oval_definitions.SetAttribute ("xmlns:unix-def", "http://oval.mitre.org/XMLSchema/oval-definitions-5#unix");
			oval_definitions.SetAttribute ("xmlns:linux-def", "http://oval.mitre.org/XMLSchema/oval-definitions-5#linux");
			oval_definitions.SetAttribute ("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
			doc.AppendChild (oval_definitions);
			
			XmlElement generator = doc.CreateElement ("generator");
			
			XmlElement product_name = doc.CreateElement ("oval:product_name", "http://oval.mitre.org/XMLSchema/oval-common-5");
			product_name.InnerText = Defines.PACKAGE;
			generator.AppendChild (product_name);
			
			XmlElement product_version = doc.CreateElement ("oval:product_version", "http://oval.mitre.org/XMLSchema/oval-common-5");
			product_version.InnerText = Defines.VERSION;
			generator.AppendChild (product_version);

			XmlElement schema_version = doc.CreateElement ("oval:schema_version", "http://oval.mitre.org/XMLSchema/oval-common-5");
			schema_version.InnerText = "5.0";
			generator.AppendChild (schema_version);
			
			XmlElement timestamp = doc.CreateElement ("oval:timestamp", "http://oval.mitre.org/XMLSchema/oval-common-5");
			timestamp.InnerText = String.Format ("{0:yyyy-MM-ddTHH:mm:ss}", DateTime.Now);
			generator.AppendChild (timestamp);
						
			oval_definitions.AppendChild (generator);			

			XmlElement definitions = doc.CreateElement ("definitions");
			oval_definitions.AppendChild (definitions);

			XmlElement tests = doc.CreateElement ("tests");
			oval_definitions.AppendChild (tests);

			XmlElement objects = doc.CreateElement ("objects");
			oval_definitions.AppendChild (objects);

			XmlElement states = doc.CreateElement ("states");
			oval_definitions.AppendChild (states);
			
			foreach (DefinitionMetaData d in dmd) {
			
				XmlElement definition = doc.CreateElement ("definition");
				definition.SetAttribute ("id", d.ID);
				definition.SetAttribute ("version", "1");
				definition.SetAttribute ("class", "vulnerability");
				
				XmlElement metadata = doc.CreateElement ("metadata");
				
				XmlElement title = doc.CreateElement ("title");
				title.InnerText = d.Title;
				metadata.AppendChild (title);

				XmlElement affected = doc.CreateElement ("affected");
				affected.SetAttribute ("family", d.Family);
				
				XmlElement platform = doc.CreateElement ("platform");
				platform.InnerText = d.Platform;
				affected.AppendChild (platform);
				
				XmlElement product = doc.CreateElement ("product");
				product.InnerText = d.Product;
				affected.AppendChild (product);
				
				metadata.AppendChild (affected);
				
				XmlElement reference = doc.CreateElement ("reference");
				reference.SetAttribute ("source", d.RefSource);
				reference.SetAttribute ("ref_id", d.RefID);
				reference.SetAttribute ("ref_url", d.RefUrl);
				metadata.AppendChild (reference);
				
				XmlElement description = doc.CreateElement ("description");
				description.InnerText = d.Description;
				metadata.AppendChild (description);
				
				definition.AppendChild (metadata);
				
				XmlElement criteria = doc.CreateElement ("criteria");				
				criteria.SetAttribute ("operator", d.CriteriaOperator.ToUpper());
				definition.AppendChild (criteria);
				
				int count = 1;
				
				foreach (TestMetaData td in tmd) {
					foreach (string s in d.TestCriteria)
						if (td.ID == s) {
							XmlElement criterion = doc.CreateElement ("criterion");
							criterion.SetAttribute ("test_ref", td.ID);
							criterion.SetAttribute ("comment", td.Comment);
							criteria.AppendChild (criterion);
						}
						
					XmlElement newTest = doc.CreateElement (td.TestType);
					newTest.SetAttribute ("id", td.ID);
					newTest.SetAttribute ("version", "1");
					newTest.SetAttribute ("check", td.Check.ToLower());
					newTest.SetAttribute ("comment", td.Comment);
					
					XmlElement objRef = doc.CreateElement ("object");
					objRef.SetAttribute ("object_ref", td.ID + ":obj:" + count.ToString());
					newTest.AppendChild (objRef);
					
					XmlElement stateRef = doc.CreateElement ("state");
					stateRef.SetAttribute ("state_ref", td.ID + ":ste:" + count.ToString());
					newTest.AppendChild (stateRef);
					
					tests.AppendChild (newTest);

					char[] splitter  = {'_'};
					string[] tmp;
					tmp = td.TestType.Split (splitter);
					string testType = tmp[0];
					
					XmlElement newObject = doc.CreateElement (testType + "_object");
					newObject.SetAttribute ("id", td.ID + ":obj:" + count.ToString());
					newObject.SetAttribute ("version", "1");
					newObject.SetAttribute ("xmlns", "");
					
					foreach (string s in td.TestObjects.Keys) {
						XmlElement objItem = doc.CreateElement (s);
						objItem.InnerText = (string) td.TestObjects[s];
						newObject.AppendChild (objItem);
					}
					
					objects.AppendChild (newObject);					

					XmlElement newState = doc.CreateElement (testType + "_state");
					newState.SetAttribute ("id", td.ID + ":ste:" + count.ToString());
					newState.SetAttribute ("version", "1");
					newState.SetAttribute ("xmlns", "");
					
					foreach (string s in td.TestStates.Keys) {
						XmlElement steItem = doc.CreateElement (s);
						steItem.InnerText = (string) td.TestStates[s];
						newState.AppendChild (steItem);
					}
					
					states.AppendChild (newState);
					
					count++;
				}
				
				definitions.AppendChild (definition);
			}
			
			return doc;
		}

		public void OnOpenActivate (object o, EventArgs args)
		{
			FileChooserDialog fcd = new FileChooserDialog (
				"Open OVAL definitions...",
				Gtk.Stock.Open, 
				mainWindow, 
				FileChooserAction.Open);

			fcd.AddButton (Gtk.Stock.Cancel, ResponseType.Cancel);
			fcd.AddButton (Gtk.Stock.Open, ResponseType.Ok);

			fcd.SelectMultiple = false;

			ResponseType response = (ResponseType) fcd.Run();
			if (response == ResponseType.Ok)  {		

//				currentTestObjects.Clear ();
//				currentTestStates.Clear ();
//			
//				fileTreeView.AppendColumn ("fileRoot", new CellRendererText (), "text", 0);
//				rootIter = fileStore.AppendValues (fcd.Filename);			
//				defIter = fileStore.AppendValues (rootIter, "Definitions");
//				testIter = fileStore.AppendValues (rootIter, "Tests");			
//				fileTreeView.ExpandAll ();
//			
//				fileTreeView.Sensitive = true;

				MessageDialog md = new MessageDialog (mainWindow, DialogFlags.DestroyWithParent, 
						MessageType.Info, ButtonsType.Close, "Not Yet Implemented");
						
				md.Run ();
				md.Destroy ();		
				
//				mainWindow.Title = String.Format ("{0} - sussen-editor", fcd.Filename);
			}
			
			fcd.Destroy ();		
		}

		public void OnSaveActivate (object o, EventArgs args)
		{
			FileChooserDialog fcd = new FileChooserDialog (
				"Save definitions to...",
				Gtk.Stock.Save, 
				mainWindow, 
				FileChooserAction.Save);

			fcd.AddButton (Gtk.Stock.Cancel, ResponseType.Cancel);
			fcd.AddButton (Gtk.Stock.Save, ResponseType.Ok);

			fcd.SelectMultiple = false;

			ResponseType response = (ResponseType) fcd.Run();
			if (response == ResponseType.Ok)  {		
		
				DefinitionMetaData[] dmd = (DefinitionMetaData[]) currentDefinitions.ToArray (typeof (DefinitionMetaData));
				TestMetaData[] tmd = (TestMetaData[]) currentTests.ToArray (typeof (TestMetaData));
			
				XmlDocument doc = GenerateDefinitionXml (dmd, tmd);
				doc.Save (fcd.Filename);
				
				fileStore.SetValue (rootIter, 0, fcd.Filename);
				mainWindow.Title = String.Format ("{0} - sussen-editor", fcd.Filename);
			}
			
			fcd.Destroy ();
		}
	
		public void OnExecuteActivate (object o, EventArgs args)
		{
			if (noteBook.CurrentPage == 0) {
				ArrayList tmp1 = new ArrayList ();
				ArrayList tmp2 = new ArrayList ();
				
				foreach (DefinitionMetaData dd in currentDefinitions) {
					if (dd.ID == defIDEntry.Text)
						tmp1.Add (dd);
						
					foreach (TestMetaData td in currentTests)
						foreach (string s in dd.TestCriteria)
							if (td.ID == s)
								tmp2.Add (td);					
				}
			
				DefinitionMetaData[] dmd = (DefinitionMetaData[]) tmp1.ToArray (typeof (DefinitionMetaData));
				TestMetaData[] tmd = (TestMetaData[]) tmp2.ToArray (typeof (TestMetaData));
				XmlDocument doc = GenerateDefinitionXml (dmd, tmd);

				Sussen.Oval.Config config = new Sussen.Oval.Config ("linux");
				config.DefinitionsFile = "foo";
				if (debugMode) {
					config.LogFile = "-";
					config.LoggingLevel = LogLevel.Debug;
				}
				
				Interpreter i = new Interpreter ();
				i.CurrentConfiguration = config;
				string res = i.AnalyzeDefinition (doc.InnerXml);

				XmlDocument resDoc = new XmlDocument ();
				resDoc.LoadXml (res);	
				
				XmlNodeList nl = resDoc.DocumentElement.GetElementsByTagName ("results");
				if (!(nl.Count > 0)) {
					return;
				}
					
				XmlElement results = (XmlElement) nl[0];
				nl = results.GetElementsByTagName ("definitions");
				if (!(nl.Count > 0)) {
					return;
				}
				
				XmlElement definitions = (XmlElement) nl[0];
				foreach (XmlElement e in definitions.ChildNodes) {					
					string id = e.GetAttribute ("definition_id");
					
					if (id == defIDEntry.Text) {
					
						string defResult = e.GetAttribute ("result");
						string critResult = null;
						ArrayList critItemResults = new ArrayList ();
						
						foreach (XmlElement c in e.ChildNodes) {
							if (c.Name == "criteria") {
								critResult = c.GetAttribute ("result");
								
								foreach (XmlElement d in c.ChildNodes)
									critItemResults.Add (d.GetAttribute ("result"));
							}
						}
						
						StringBuilder sb = new StringBuilder ();
						sb.AppendFormat ("Definition result: {0}\n", defResult);
						sb.AppendFormat ("Criteria result: {0}\n", critResult);
						
						foreach (string s in critItemResults)
							sb.AppendFormat ("Criterion result: {0}\n", s);
							
						MessageDialog md = new MessageDialog (mainWindow, DialogFlags.DestroyWithParent, 
							MessageType.Info, ButtonsType.Close, sb.ToString());
						
						md.Run ();
						md.Destroy ();
					}
				}
							
			} else if (noteBook.CurrentPage == 1) {				
				foreach (TestMetaData td in currentTests) {
					if (td.ID == testIDEntry.Text) {
						
						Test test;
						test = Interpreter.CreateTest (td.TestType);
						
						string itemName = GetItemName (td.TestType) + "_object";
						
						ObjectElement oe;
						oe = new ObjectElement (itemName, td.ID, "1", "");
						
						foreach (string s in td.TestObjects.Keys) {					
							EntityObject eo = new EntityObject (
								s,
								(string) td.TestObjects[s], 
								Operations.Equals, 
								OvalDataTypes.StringType, 
								Checks.All);
								
							oe.Entities [eo.Name] = eo;
						}
						
						string res = Interpreter.RunTest (test, oe);
						
						MessageDialog md = new MessageDialog (mainWindow, DialogFlags.DestroyWithParent, 
							MessageType.Info, ButtonsType.Close, res);
				
						md.Run ();
						md.Destroy ();							
					}
				}
			}
		}

		string GetItemName (string testType)
		{
			char[] splitter  = {'_'};
			string[] res;
			
			res = testType.Split (splitter);
			return res[0];
		}

		[ConnectBefore]
		public void OnFileRightClick (object o, ButtonPressEventArgs args)
		{
			if (args.Event.Button == 3)
				DoPopUp ();
		}
		
		public void OnQuitActivate (object o, EventArgs args)
		{
			Application.Quit ();
		}

		public void OnAboutActivate (object o, EventArgs args)
		{
			string[] author = { "Loren Bandiera" };
			string[] docs = { "" };
			string translators = "";
			string desc = "OVAL definition editor";
			string copy = "Copyright \xa9 2005 MMG Security, Inc.";
			
			Gnome.About ab = new Gnome.About (
				"sussen-editor",
				Defines.VERSION,
				copy,
				desc,
				author,
				docs, 
				translators, 
				sussenLogo);

			ab.Icon = sussenIcon;
			ab.Show ();		
		}
			
		public void OnWindowDelete (object o, DeleteEventArgs args)
		{			
			Application.Quit ();
			args.RetVal = true;
		}
		
		#endregion 
	}
	
	public class AddTestDialog
	{
		Glade.XML ui;

		[Glade.Widget] Gtk.Dialog addTestDialog;
		[Glade.Widget] Gtk.TreeView testTreeView;

		ListStore store;		

		public AddTestDialog (ArrayList currentTests)
		{
			ui = new Glade.XML (null, "sussen-editor.glade", "addTestDialog", null);
			ui.Autoconnect (this);

			store = new ListStore (typeof (bool), typeof (string), typeof (string));

			CellRendererToggle crt = new CellRendererToggle();
			crt.Activatable = true;
			crt.Toggled += OnTestToggled;

			testTreeView.AppendColumn ("Enabled", crt, "active", 0);
			testTreeView.AppendColumn ("ID", new CellRendererText (), "text", 1);
			testTreeView.AppendColumn ("Description", new CellRendererText (), "text", 2);
			
			testTreeView.Model = store;

			foreach (TestMetaData tmd in currentTests) {
				store.AppendValues (false, tmd.ID, tmd.Comment);
			}
			
			addTestDialog.Resize (300, 400);
			addTestDialog.Run ();
			addTestDialog.Destroy ();
		}

		void OnTestToggled (object o, ToggledArgs args)
		{
			TreeIter iter;

			if (store.GetIter (out iter, new TreePath(args.Path))) {
				bool old = (bool) store.GetValue (iter,0);
				store.SetValue(iter,0,!old);
			}
		}

		public void OnOkClicked (object o, EventArgs args)
		{
			addTestDialog.HideAll ();
		}

		public void OnDlgDelete (object o, DeleteEventArgs args)
		{
			addTestDialog.HideAll ();
		}

		public TestMetaData[] SelectedTests
		{
			get {
				ArrayList list = new ArrayList ();
				
				foreach (object[] row in store) {
					TestMetaData tmd = new TestMetaData();
					tmd.ID = (string) row[1];
					tmd.Comment = (string) row[2];
					list.Add (tmd);
				}
				
				return (TestMetaData[]) list.ToArray (typeof(TestMetaData));
			}
		}
	}
}