// 
// VersionTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Xml;

using Sussen.Oval;

namespace Sussen.Oval.Apache
{
	public class VersionTest : Test
	{
		public const string VERSION_NAMESPACE = "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#apache";
	
		public VersionTest ()
		{
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			items.Add (CollectData ());
			
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		CollectedItem CollectData ()
		{
			CollectedItem item = new CollectedItem ("version_item", 0, VERSION_NAMESPACE, Flags.NotCollected);
			
			string[] commonPaths = { "/usr/sbin/httpd", "/usr/sbin/apache2", "c:\apache group\apache2\bin\apache.exe" };
			string output = CmdReader.Read (commonPaths, "-v");

			if (output == null) {
				item.Flag = Flags.Error;
				item.Message = "Unable to determine apache version";
				item.MessageLevel = MessageLevels.Error;
				
				item.AddItem (new ItemEntity ("version", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				return item;
			}

			char[] newlineSplit = {'\n'};
			string[] tmp = output.Split (newlineSplit);

			char[] colonSplit = {':'};
			string[] tmp2 = tmp[0].Split (colonSplit);

			char[] slashSplit = {'/'};
			string[] tmp3 = tmp2[1].Split (slashSplit);

			item.Flag = Flags.Complete;
			item.AddItem (new ItemEntity ("version", tmp3[1], OvalDataTypes.StringType, StatusFlags.Error, false));
			
			return item;
		}
	}
}
