// 
// EnvironmentVariableTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using Sussen.Oval;

namespace Sussen.Oval.Independent
{
	public class EnvironmentVariableTest : Test
	{
		int itemCount;
		
		public EnvironmentVariableTest ()
		{
			itemCount = 0;
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject varName = objectElement.Entities["name"];
			if (varName == null)
				throw new TestException (objectElement.ID, "Missing name object from environment variable test");

			if (varName.DataType != OvalDataTypes.StringType)
				throw new TestException (objectElement.ID, "Invalid datatype for name object in environment variable test");

			if (varName.VariableInstance != null) {
			
				foreach (string v in varName.VariableInstance.ValueArray) {
					string[] names = GetVariableNames (v, varName.Operation);
					foreach (string n in names) {
						items.Add (CollectData (n));
						itemCount++;
					}				
				}
			
			} else {
			
				string[] names = GetVariableNames (varName.Value, varName.Operation);
				foreach (string n in names) {
					items.Add (CollectData (n));
					itemCount++;
				}
			}

			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		string[] GetVariableNames (string name, Operations operation)
		{		
			ArrayList items = new ArrayList ();

			switch (operation) {
			
			case Operations.Equals:
				items.Add (name);
				break;

			case Operations.NotEqual:
			case Operations.PatternMatch:				
				return FindVariables (name, operation);
				
			default:
				throw new TestException ("", "Invalid operation for name object in environmentvariable test");
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		string[] FindVariables (string varName, Operations operation)
		{
			ArrayList items = new ArrayList ();

			try {
				
				IDictionary vars = Environment.GetEnvironmentVariables();
				foreach (DictionaryEntry de in vars) {
					string name = (string) de.Key;
					if (CompareItems (varName, name, operation))
						items.Add (name);
				}
			
			} catch (Exception e) {			
				Log.Debug (e);
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		CollectedItem CollectData (string varName)
		{
			CollectedItem item = new CollectedItem ("environmentvariable_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.Complete);
			
			try {

				string evValue = Environment.GetEnvironmentVariable (varName);
		
				item.AddItem (new ItemEntity ("name", varName, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new ItemEntity ("value", evValue, OvalDataTypes.StringType, StatusFlags.Exists, false));

			} catch (Exception e) {
				
				item.Flag = Flags.Error;
				item.Message = e.Message;
				item.MessageLevel = MessageLevels.Error;
				
				item.AddItem (new ItemEntity ("name", varName, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new ItemEntity ("value", "", OvalDataTypes.StringType, StatusFlags.Error, false));
			}			

			return item;
		}
	}
}
