// 
// FileMD5Test.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Xml;
using Sussen.Oval;

namespace Sussen.Oval.Independent
{
	public class FileMD5Test : Test
	{
		int itemCount;
		
		public FileMD5Test ()
		{
			itemCount = 0;
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject path = objectElement.Entities["path"];
			EntityObject file = objectElement.Entities["filename"];

			if (path == null || file == null)
				throw new TestException (objectElement.ID, "Invalid filemd5 test");

			// If the nillable attribute is set to true, then the object being specified is the higher 
			// level path.  In this case, the filename element should not be collected or used in analysis.
			if (file.Nil) {
				// FIXME: does that mean md5sum a whole directory? error in schema docs?
			} else {

				FileBehaviors fb = new FileBehaviors (objectElement.Behaviors);
				FileCrawler fc = new FileCrawler (path, file, fb);
				fc.Run ();
			
				foreach (string f in fc.FoundFiles) {
					items.Add (CollectData (f));
					itemCount++;
				}
			}
			
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		CollectedItem CollectData (string fullName)
		{
			CollectedItem item = new CollectedItem ("filemd5_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.Complete); 

			string filePath, fileName;			
			FileInfo tmpFi = new FileInfo (fullName);
			filePath = tmpFi.Directory.FullName;
			fileName = tmpFi.Name;

			try {
				string fileHashString = null;
				byte[] fileHash = null;

				FileStream fs = File.OpenRead (fullName);

				MD5 md5sum = new MD5CryptoServiceProvider();

				fileHash = md5sum.ComputeHash (fs);
				fileHashString = BitConverter.ToString(fileHash).Replace("-", "");
				fileHashString = fileHashString.ToLower ();

				fs.Close ();

				item.AddItem (new ItemEntity ("path", filePath, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new ItemEntity ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new ItemEntity ("md5", fileHashString, OvalDataTypes.StringType, StatusFlags.Exists, false));

			} catch (Exception e) {

				item.Flag = Flags.Error;
				item.Message = e.Message;
				item.MessageLevel = MessageLevels.Error;
				
				item.AddItem (new ItemEntity ("path", filePath, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new ItemEntity ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new ItemEntity ("md5", "", OvalDataTypes.StringType, StatusFlags.Error, false));

			}			
			
			return item;
		}
	}
}
