// 
// TextFileContentTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Xml;

namespace Sussen.Oval.Independent
{
	public class TextFileContentTest : Test
	{
		int itemCount;
		
		public TextFileContentTest ()
		{
			itemCount = 0;
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject path = objectElement.Entities["path"];
			EntityObject file = objectElement.Entities["filename"];
			EntityObject line = objectElement.Entities["line"];

			if (path == null || file == null || line == null)
				throw new TestException (objectElement.ID, "Invalid textfilecontent test");

			FileBehaviors fb = new FileBehaviors (objectElement.Behaviors);
			FileCrawler fc = new FileCrawler (path, file, fb);
			fc.Run ();
			
			foreach (string f in fc.FoundFiles) {
				items.Add (CollectData (f, line.Value));
				itemCount++;
			}
			
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		CollectedItem CollectData (string fullName, string fileLine)
		{
			CollectedItem item = new CollectedItem ("textfilecontent_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.Complete);

			string filePath, fileName;			
			FileInfo tmpFi = new FileInfo (fullName);
			filePath = tmpFi.Directory.FullName;
			fileName = tmpFi.Name;
			
			try {

				StreamReader sr = new StreamReader (fullName);
				int line = int.Parse (fileLine);
				int lineCount = 0;
				string txt = null;

				while ((txt = sr.ReadLine()) != null) {
					if (lineCount == line) {
						
						item.AddItem (new ItemEntity ("path", filePath, OvalDataTypes.StringType, StatusFlags.Exists, true));
						item.AddItem (new ItemEntity ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Exists, true));
						item.AddItem (new ItemEntity ("line", fileLine, OvalDataTypes.StringType, StatusFlags.Exists, true));
						item.AddItem (new ItemEntity ("subexpression", txt, OvalDataTypes.StringType, StatusFlags.Exists, false));

						break;
					}

					lineCount++;
				}

				sr.Close ();

			} catch (Exception e) {

				item.Flag = Flags.Error;
				item.Message = e.Message;
				item.MessageLevel = MessageLevels.Error; 

				item.AddItem (new ItemEntity ("path", filePath, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new ItemEntity ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new ItemEntity ("line", fileLine, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new ItemEntity ("subexpression", "", OvalDataTypes.StringType, StatusFlags.Error, false));
			}
			
			return item;
		}
	}
}
