// 
// VariableTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

namespace Sussen.Oval.Independent
{
	public class VariableTest : Test
	{
		int itemCount;
		
		public VariableTest ()
		{
			itemCount = 0;
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject varName = objectElement.Entities["var_ref"];
			if (varName == null)
				throw new TestException (objectElement.ID, "Missing name object from variable test");

			if (varName.DataType != OvalDataTypes.StringType)
				throw new TestException (objectElement.ID, "Invalid datatype for name object in variable test");

			switch (varName.Operation) {
			
			case Operations.Equals:
				items.Add (CollectData (varName.Value));
				break;
				
			case Operations.NotEqual:
			case Operations.PatternMatch:
				string[] foundVars = FindVariables (varName);
				foreach (string v in foundVars) {
					items.Add (CollectData (v));
					itemCount++;
				}
				break;
			
			default:
				throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (varName.Operation));
			}
			
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		string[] FindVariables (EntityObject varName)
		{
			ArrayList items = new ArrayList ();
		
			foreach (DictionaryEntry de in Global.OvalVariables) {
				Variable v = (Variable) de.Value;
				if (CompareItems (varName.Value, v.ID, varName.Operation))
					items.Add (v.ID);
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		CollectedItem CollectData (string varName)
		{
			CollectedItem item = new CollectedItem ("variable_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.Complete);
			
			Variable v = Global.OvalVariables [varName];
			if (v == null) {
				item.AddItem (new ItemEntity ("var_ref", varName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("value", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			} else {
				item.AddItem (new ItemEntity ("var_ref", varName, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new ItemEntity ("value", v.Value, v.DataType, StatusFlags.Exists, false));			
			}
			
			return item;
		}
	}
}
