// 
// XmlFileContentTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Xml;
using System.Xml.XPath;

namespace Sussen.Oval.Independent
{
	public class XmlFileContentTest : Test
	{
		int itemCount;
		
		public XmlFileContentTest ()
		{
			itemCount = 0;
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject path = objectElement.Entities["path"];
			EntityObject file = objectElement.Entities["filename"];
			EntityObject xpath = objectElement.Entities["xpath"];

			if (path == null || file == null || xpath == null)
				throw new TestException (objectElement.ID, "Invalid xmlfilecontent test");

			FileBehaviors fb = new FileBehaviors (objectElement.Behaviors);
			FileCrawler fc = new FileCrawler (path, file, fb);
			fc.Run ();
			
			foreach (string f in fc.FoundFiles) {
				items.Add (CollectData (f, xpath.Value));
				itemCount++;
			}
			
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		CollectedItem CollectData (string fullName, string fileXpath)
		{
			CollectedItem item = new CollectedItem ("xmlfilecontent_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.Complete);

			string filePath, fileName;			
			FileInfo tmpFi = new FileInfo (fullName);
			filePath = tmpFi.Directory.FullName;
			fileName = tmpFi.Name;
			
			try {

				XPathNavigator nav;
				XPathDocument doc;
				XPathNodeIterator NodeIter;

				doc = new XPathDocument (fullName);
				nav = doc.CreateNavigator();

				NodeIter = nav.Select(fileXpath);
				while (NodeIter.MoveNext()) {
					
					item.AddItem (new ItemEntity ("path", filePath, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new ItemEntity ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new ItemEntity ("xpath", fileXpath, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new ItemEntity ("value_of", NodeIter.Current.Value, OvalDataTypes.StringType, StatusFlags.Exists, false));
				}

			} catch (Exception e) {

				item.Flag = Flags.Error;
				item.Message = e.Message;
				item.MessageLevel = MessageLevels.Error;
				
				item.AddItem (new ItemEntity ("path", filePath, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new ItemEntity ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new ItemEntity ("xpath", fileXpath, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new ItemEntity ("value_of", "", OvalDataTypes.StringType, StatusFlags.Error, false));
			}
			
			return item;
		}	
	}
}
