// 
// PasswordTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using Mono.Unix;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public class PasswordTest : Test
	{
		int itemCount;
		
		public PasswordTest ()
		{
			itemCount = 0;
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject userName = objectElement.Entities["username"];
			if (userName == null)
				throw new TestException (objectElement.ID, "Missing name object from password test");

			if (userName.DataType != OvalDataTypes.StringType)
				throw new TestException (objectElement.ID, "Invalid datatype for name object in password test");				

			if (userName.VariableInstance != null) {
			
				foreach (string v in userName.VariableInstance.ValueArray) {
					string[] names = GetUserNames (v, userName.Operation);
					foreach (string n in names) {
						items.Add (CollectData (n));
						itemCount++;
					}				
				}
			
			} else {
			
				string[] names = GetUserNames (userName.Value, userName.Operation);
				foreach (string n in names) {
					items.Add (CollectData (n));
					itemCount++;
				}
			}
	
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		string[] GetUserNames (string name, Operations operation)
		{		
			ArrayList items = new ArrayList ();

			switch (operation) {
			
			case Operations.Equals:
				items.Add (name);
				break;

			case Operations.NotEqual:
			case Operations.PatternMatch:				
				return FindUsers (name, operation);
				
			default:
				throw new TestException ("", "Invalid operation for username object in password test");
			}
			
			return (string[]) items.ToArray (typeof (string));
		}

		string[] FindUsers (string userName, Operations operation)
		{
			ArrayList items = new ArrayList ();

			UnixUserInfo[] userList = UnixUserInfo.GetLocalUsers ();
			foreach (UnixUserInfo u in userList)
				if (CompareItems (userName, u.UserName, operation))
					items.Add (u);
			
			return (string[]) items.ToArray (typeof (string));
		}

		CollectedItem CollectData (string userName)
		{
			CollectedItem item = new CollectedItem ("password_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.NotCollected);

			try {

				UnixUserInfo ui = new UnixUserInfo (userName);

				item.Flag = Flags.Complete;
				item.AddItem (new ItemEntity ("username", userName, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new ItemEntity ("password", ui.Password, OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("user_id", ui.UserId.ToString(), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("group_id", ui.GroupId.ToString(), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("gcos", ui.RealName, OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("home_dir", ui.HomeDirectory, OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new ItemEntity ("login_shell", ui.ShellProgram, OvalDataTypes.StringType, StatusFlags.Exists, false));
								
				return item;

			} catch (Exception e){

				item.Flag = Flags.Complete;				
				item.AddItem (new ItemEntity ("username", userName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new ItemEntity ("password", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("user_id", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("group_id", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("gcos", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("home_dir", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new ItemEntity ("login_shell", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				
				return item;
			}						
		}
	}
}
