// 
// SccsTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public class SccsTest : Test
	{
		public SccsTest ()
		{
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			ArrayList items = new ArrayList ();
			
			EntityObject path = objectElement.Entities["path"];
			EntityObject file = objectElement.Entities["filename"];

			if (path == null || file == null)
				throw new TestException (objectElement.ID, "Invalid sccs test");

			// FIXME: handle more than more item			
			items.Add (CollectData (path.Value, file.Value));
			
			return (CollectedItem[]) items.ToArray (typeof (CollectedItem));
		}

		CollectedItem CollectData (string filePath, string fileName)
		{
			CollectedItem item = new CollectedItem ("sccs_item", 0, Vocabulary.UNIX_NAMESPACE, Flags.NotCollected);
			item.AddItem (new ItemEntity ("path", filePath, OvalDataTypes.StringType, StatusFlags.NotCollected, true));
			item.AddItem (new ItemEntity ("filename", fileName, OvalDataTypes.StringType, StatusFlags.NotCollected, true));		
			item.AddItem (new ItemEntity ("module_name", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new ItemEntity ("module_type", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new ItemEntity ("release", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new ItemEntity ("level", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new ItemEntity ("branch", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new ItemEntity ("sequence", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new ItemEntity ("what_string", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			
			return item;
		}		
	}
}
