// 
// Config.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Diagnostics;
using System.IO;
using System.Net;

namespace Sussen.Oval
{
	public enum ResultOutputTypes {
		HTML,
		XML
	};

	// FIXME: need a better way
	public class Global
	{
		public Global ()
		{
		}
		
		public static VariableContainer OvalVariables;
		public static StateContainer SystemObjects;
	}

	public class Config
	{
		public string DefinitionsFile;
		public string DefinitionsFileHash;
		public string SystemCharacteristicsFile;
		public string ResultsFile;
		public ResultOutputTypes ResultsFileFormat;
		
		public string LogFile;
		public LogLevel LoggingLevel;
		
		public string FamilyName;
		public FamilyTypes FamilyType;

		public bool VerifyMD5;
		public bool VerifyXML;

		public bool AnalyzeOnly;
		public bool CollectOnly;

		public string XmlDirectory;
		public string DownloadDirectory;
		
		public Config (string familyName)
		{
			this.DefinitionsFile = "";
			this.DefinitionsFileHash = "";
			this.SystemCharacteristicsFile = "system-characteristics.xml";
			this.ResultsFile = "results.xml";
			this.ResultsFileFormat = ResultOutputTypes.XML;
			
			this.LogFile = "";
			this.LoggingLevel = LogLevel.Info;

			this.FamilyName = familyName;
			this.FamilyType = Vocabulary.GetFamilyType (familyName);

			this.VerifyMD5 = false;
			this.VerifyXML = true;
			
			this.CollectOnly = false;
			this.AnalyzeOnly = false;

			this.XmlDirectory = "";
			this.DownloadDirectory = "";
		}
		
		public string GetXmlResource (string relativePathName)
		{
			string retVal = "";
		
			if (this.XmlDirectory.StartsWith ("http")) {
			
				string downloadPath = System.IO.Path.Combine (this.DownloadDirectory, relativePathName);
				
				DirectoryInfo di = new DirectoryInfo (this.DownloadDirectory);
				if (!di.Exists)
					di.Create ();
				
				// FIXME: doesn't take into account if file changed since being downloaded.
				if (File.Exists (downloadPath)) {
				
					retVal = downloadPath;
					
				} else {
				
					string url = System.IO.Path.Combine (this.XmlDirectory, relativePathName);
					
					// FIXME: Should be able to use credentials to access resource
					WebClient wc = new WebClient ();
					wc.DownloadFile (url, downloadPath);
					
					retVal = downloadPath;
				}
			
			} else {
				retVal = System.IO.Path.Combine (this.XmlDirectory, relativePathName);
			}
			
			return retVal;
		}
	}
}
