// 
// DataCollector.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Xml;

namespace Sussen.Oval
{
	public delegate void PlatformCreateTest (TestElement testElement, ObjectElement objectElement);

	public class DataCollector
	{
		OvalDocuments docs;
		ArrayList tdItems;
		int tdGUID = 1;

		public PlatformCreateTest CreateTest;		

		public DataCollector (OvalDocuments ovalDocuments)
		{
			if (ovalDocuments == null)
				throw new ArgumentNullException ("ovalDocuments");
				
			docs = ovalDocuments;
			tdItems = new ArrayList ();
		}
	
		public void RunTest (Test test, TestElement testElement, ObjectElement objectElement)
		{			
			try {
				
				CollectedItem[] results;
				results = test.Run (testElement, objectElement);
				
				foreach (CollectedItem ci in results) {			
				
					Log.Debug ("Ran test {0} got status {1}", testElement.ID, Vocabulary.GetFlag (ci.Flag));
				
					if (!this.tdItems.Contains (objectElement.ID)) {
					
						tdItems.Add (objectElement.ID);			
						ci.ID = tdGUID++;

						// FIXME: verify node can have message and reference			 
						if (ci.Message != null)
							docs.SystemCharacteristics.WriteCollectedObject (objectElement.ID, objectElement.Version, ci.Flag, ci.Message, ci.MessageLevel);
						else
							docs.SystemCharacteristics.WriteCollectedObject (objectElement.ID, objectElement.Version, ci.Flag, ci.ID);
					
						ci.Write (docs.SystemCharacteristics);
					}
				}
				
			} catch (TestException te) {
				// FIXME: should be able to use te.ID but it's always blank
				Log.Warn ("Unable to run test [{0}]. {1}", testElement.ID, te.Message);
				Log.Debug (te);
			}
		}

		public void Run ()
		{
			if (CreateTest == null)
				throw new ArgumentNullException ("CreateTest");
	
			foreach (DictionaryEntry de in docs.Definitions.Tests) {				
				TestElement te = (TestElement) de.Value;
				if (te.ObjectRef == null)
                    continue;
                    
				ObjectElement oe = docs.Definitions.Objects [te.ObjectRef];
				CreateTest (te, oe);					
			}
		}		
	}
}
