// 
// Vocabulary.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;

namespace Sussen.Oval
{
	public enum Checks {
		All,
		AtLeastOne,
		NoneExist,
		OnlyOne
	};

	public enum FamilyTypes { 
		IOS, 
		Linux,
		MacOS,
		Unix,
		Windows,
		Unknown
	};

	public enum Flags {
		Error,
		Complete,
		Incomplete,
		DoesNotExist,
		NotCollected,
		NotApplicable
	};

	public enum MessageLevels {
		Debug,
		Error,
		Fatal,
		Info,
		Warning
	};
	
	public enum Operations {
		Equals,
		NotEqual,
		GreaterThan,
		LessThan,
		GreaterThanOrEqual,
		LessThanOrEqual,
		BitwiseAnd,
		BitwiseOr,
		PatternMatch
	};

	public enum OvalDataTypes {
		BinaryType,
		BooleanType,
		EvrStringType,
		FloatType,
		IOSVersionType,
		IntegerType,
		StringType,
		VersionType
	};
	
	public enum StatusFlags {
		Error,
		Exists,
		DoesNotExist,
		NotCollected
	};

	public enum TestResult
	{
		True,
		False,
		Error,
		Unknown,
		NotEvaluated,
		NotApplicable
	};
	
	public enum VariableTypes {
		External,
		Constant,
		Local
	};
	
	public static class Vocabulary
	{
		public const string APACHE_NAMESPACE = "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#apache";
		public const string LINUX_NAMESPACE = "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#linux";
		public const string INDEPENDENT_NAMESPACE = "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#independent";
		public const string UNIX_NAMESPACE = "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#unix";
		public const string WINDOWS_NAMESPACE = "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#windows";
	
		public const string DEFINITION_ID_PATTERN = @"oval:[A-Za-z0-9_\-\.]+:def:[1-9][0-9]*";
		public const string OBJECT_ID_PATTERN = @"oval:[A-Za-z0-9_\-\.]+:obj:[1-9][0-9]*";
		public const string STATE_ID_PATTERN = @"oval:[A-Za-z0-9_\-\.]+:ste:[1-9][0-9]*";
		public const string TEST_ID_PATTERN = @"oval:[A-Za-z0-9_\-\.]+:tst:[1-9][0-9]*";
		public const string VARIABLE_ID_PATTERN = @"oval:[A-Za-z0-9_\-\.]+:var:[1-9][0-9]*";
	
		public static Checks GetCheck (string checkName)
		{
			if (checkName == null)
				checkName = "";
		
			switch (checkName.ToLower()) {

			case "":
			case "all":
				return Checks.All;
				
			case "at least one":
				return Checks.AtLeastOne;

			case "none exist":
				return Checks.NoneExist;
				
			case "only one":
				return Checks.OnlyOne;
			
			default:
				throw new ArgumentOutOfRangeException (checkName);
			}		
		}	

		public static string GetCheck (Checks checkType)
		{
			string retVal = "";
			
			switch (checkType) {
			
			case Checks.All:
				retVal = "all";
				break;
				
			case Checks.AtLeastOne:
				retVal = "at least one";
				break;
				
			case Checks.NoneExist:
				retVal = "none exist";
				break;
				
			case Checks.OnlyOne:
				retVal = "only one";
				break;
											
			default:
				throw new ArgumentOutOfRangeException ();
			}
			
			return retVal;		
		}	
	
		public static OvalDataTypes GetDataType (string dataTypeName)
		{
			if (dataTypeName == null)
				dataTypeName = "";
		
			switch (dataTypeName.ToLower()) {

			case "binary":
				return OvalDataTypes.BinaryType;
				
			case "boolean":
				return OvalDataTypes.BooleanType;

			case "evr_string":
				return OvalDataTypes.EvrStringType;
				
			case "float":
				return OvalDataTypes.FloatType;

			case "ios_version":
				return OvalDataTypes.IOSVersionType;
				
			case "int":
				return OvalDataTypes.IntegerType;

			case "":
			case "string":
				return OvalDataTypes.StringType;
				
			case "version":
				return OvalDataTypes.VersionType;
				
			default:
				throw new ArgumentOutOfRangeException (dataTypeName);
			}		
		}

		public static string GetDataType (OvalDataTypes dataType)
		{
			string retVal = "";
			
			switch (dataType) {
			
			case OvalDataTypes.BinaryType:
				retVal = "binary";
				break;
				
			case OvalDataTypes.BooleanType:
				retVal = "boolean";
				break;
				
			case OvalDataTypes.EvrStringType:
				retVal = "evr_string";
				break;
				
			case OvalDataTypes.FloatType:
				retVal = "float";
				break;
				
			case OvalDataTypes.IOSVersionType:
				retVal = "ios_version";
				break;
				
			case OvalDataTypes.IntegerType:
				retVal = "int";
				break;
				
			case OvalDataTypes.StringType:
				retVal = "string";
				break;
				
			case OvalDataTypes.VersionType:
				retVal = "version";
				break;
							
			default:
				throw new ArgumentOutOfRangeException ();
			}
			
			return retVal;		
		}

		public static Flags GetFlag (string flagName)
		{
			if (flagName == null)
				flagName = "";
		
			switch (flagName.ToLower()) {

			case "error":
				return Flags.Error;
				
			case "complete":
				return Flags.Complete;

			case "incomplete":
				return Flags.Incomplete;

			case "does not exist":
				return Flags.DoesNotExist;
				
			case "not collected":
				return Flags.NotCollected;

			case "not applicable":
				return Flags.NotApplicable;
							
			default:
				throw new ArgumentOutOfRangeException (flagName);
			}		
		}

		public static string GetFlag (Flags flag)
		{
			string retVal = "";
			
			switch (flag) {
			
			case Flags.Error:
				retVal = "error";
				break;
				
			case Flags.Complete:
				retVal = "complete";
				break;
				
			case Flags.Incomplete:
				retVal = "incomplete";
				break;
				
			case Flags.DoesNotExist:
				retVal = "does not exist";
				break;
				
			case Flags.NotCollected:
				retVal = "not collected";
				break;
				
			case Flags.NotApplicable:
				retVal = " not applicable";
				break;
							
			default:
				throw new ArgumentOutOfRangeException ();
			}
			
			return retVal;		
		}

		public static MessageLevels GetMessageLevel (string levelName)
		{
			if (levelName == null)
				levelName = "";
		
			switch (levelName.ToLower()) {

			case "debug":
				return MessageLevels.Debug;
				
			case "error":
				return MessageLevels.Error;

			case "fatal":
				return MessageLevels.Fatal;

			case "info":
				return MessageLevels.Info;
				
			case "warning":
				return MessageLevels.Warning;
							
			default:
				throw new ArgumentOutOfRangeException (levelName);
			}		
		}

		public static string GetMessageLevel (MessageLevels level)
		{
			string retVal = "";
			
			switch (level) {
			
			case MessageLevels.Debug:
				retVal = "debug";
				break;
				
			case MessageLevels.Error:
				retVal = "error";
				break;
				
			case MessageLevels.Fatal:
				retVal = "fatal";
				break;
				
			case MessageLevels.Info:
				retVal = "info";
				break;
				
			case MessageLevels.Warning:
				retVal = "warning";
				break;
				
			default:
				throw new ArgumentOutOfRangeException ();
			}
			
			return retVal;		
		}
	
		public static string GetOperation (Operations operation)
		{
			string retVal = "";
			
			switch (operation) {
			
			case Operations.Equals:
				retVal = "equals";
				break;
				
			case Operations.NotEqual:
				retVal = "not equal";
				break;
				
			case Operations.GreaterThan:
				retVal = "greater than";
				break;
				
			case Operations.LessThan:
				retVal = "less than";
				break;
				
			case Operations.GreaterThanOrEqual:
				retVal = "greater than or equal";
				break;
				
			case Operations.LessThanOrEqual:
				retVal = "less than or equal";
				break;
				
			case Operations.BitwiseAnd:
				retVal = "bitwise and";
				break;
				
			case Operations.BitwiseOr:
				retVal = "bitwise or";
				break;
				
			case Operations.PatternMatch:
				retVal = "pattern match";
				break;
				
			default:
				throw new ArgumentOutOfRangeException ();
			}
			
			return retVal;
		}
	
		public static Operations GetOperation (string operationName)
		{
			if (operationName == null)
				operationName = "";
		
			switch (operationName.ToLower()) {
			
			case "":
			case "equals":
				return Operations.Equals;
				
			case "not equal":
				return Operations.NotEqual;

			case "greater than":
				return Operations.GreaterThan;
				
			case "less than":
				return Operations.LessThan;

			case "greater than or equal":
				return Operations.GreaterThanOrEqual;
				
			case "less than or equal":
				return Operations.LessThanOrEqual;

			case "bitwise and":
				return Operations.BitwiseAnd;
				
			case "bitwise or":
				return Operations.BitwiseOr;
				
			case "pattern match":
				return Operations.PatternMatch;
				
			default:
				throw new ArgumentOutOfRangeException (operationName);
			}
		}
		
		public static string GetStatusFlag (StatusFlags flag)
		{
			string retVal = "";
			
			switch (flag) {
			
			case StatusFlags.Error:
				retVal = "error";
				break;
				
			case StatusFlags.Exists:
				retVal = "exists";
				break;
				
			case StatusFlags.DoesNotExist:
				retVal = "does not exist";
				break;
				
			case StatusFlags.NotCollected:
				retVal = "not collected";
				break;
							
			default:
				throw new ArgumentOutOfRangeException ();
			}
			
			return retVal;
		}
	
		public static StatusFlags GetStatusFlag (string flagName)
		{
			if (flagName == null)
				flagName = "";
		
			switch (flagName.ToLower()) {
			
			case "error":
				return StatusFlags.Error;

			case "":				
			case "exists":
				return StatusFlags.Exists;

			case "does not exist":
				return StatusFlags.DoesNotExist;
				
			case "not collected":
				return StatusFlags.NotCollected;
				
			default:
				throw new ArgumentOutOfRangeException (flagName);
			}
		}
		
		public static string GetTestResult (TestResult testResult)
		{
			string retVal = "unknown";

			switch (testResult) {

			case TestResult.True:
				retVal = "true";
				break;
				
			case TestResult.False:
				retVal = "false";
				break;

			case TestResult.Error:
				retVal = "error";
				break;
				
			case TestResult.Unknown:
				retVal = "unknown";
				break;
				
			case TestResult.NotEvaluated:
				retVal = "not evaluated";
				break;
				
			case TestResult.NotApplicable:
				retVal = "not applicable";
				break;
				
			default:
				break;
			}
			
			return retVal;
		}
		
		public static string GetVariableType (VariableTypes variableType)
		{
			string retVal = "";
			
			switch (variableType) {
			
			case VariableTypes.Constant:
				retVal = "constant_variable";
				break;
				
			case VariableTypes.External:
				retVal = "external_variable";
				break;
				
			case VariableTypes.Local:
				retVal = "local_variable";
				break;
							
			default:
				throw new ArgumentOutOfRangeException ();
			}
			
			return retVal;
		}
	
		public static VariableTypes GetVariableType (string variableName)
		{
			if (variableName == null)
				variableName = "";
		
			switch (variableName.ToLower()) {
			
			case "constant_variable":
				return VariableTypes.Constant;

			case "external_variable":
				return VariableTypes.External;

			case "local_variable":
				return VariableTypes.Local;
							
			default:
				throw new ArgumentOutOfRangeException (variableName);
			}
		}
		
		public static string GetFamilyType (FamilyTypes familyType)
		{
			string retVal = "";
			
			switch (familyType) {
			
			case FamilyTypes.IOS:
				retVal = "ios";
				break;
				
			case FamilyTypes.Linux:
				retVal = "linux";
				break;
				
			case FamilyTypes.MacOS:
				retVal = "macos";
				break;
			
			case FamilyTypes.Unix:
				retVal = "unix";
				break;
				
			case FamilyTypes.Windows:
				retVal = "windows";
				break;
							
			default:
				throw new ArgumentOutOfRangeException ();
			}
			
			return retVal;
		}
	
		public static FamilyTypes GetFamilyType (string familyName)
		{
			if (familyName == null)
				familyName = "";
		
			switch (familyName.ToLower()) {

			case "ios":
				return FamilyTypes.IOS;
				
			case "linux":
				return FamilyTypes.Linux;
				
			case "macos":
				return FamilyTypes.MacOS;
			
			case "unix":
				return FamilyTypes.Unix;
				
			case "windows":
				return FamilyTypes.Windows;
							
			default:
				throw new ArgumentOutOfRangeException (familyName);
			}
		}		
	}
}