//
// AnalyzeByCheckTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Collections;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Unix;

namespace SussenTests.Oval.Unix
{
	[TestFixture]
	public class AnalyzeByCheckTest
	{
		[SetUp]
		public void SetUp()
		{
		}

		[TearDown]
		public void TearDown ()
		{
		}

		public ArrayList CreateResults (int trueCount, int falseCount, int errorCount, int unknownCount, int notEvaluatedCount, int notApplicableCount)
		{
			ArrayList results = new ArrayList ();

			for (int i = 0; i < trueCount; i++)
				results.Add (TestResult.True);

			for (int i = 0; i < falseCount; i++)
				results.Add (TestResult.False);

			for (int i = 0; i < errorCount; i++)
				results.Add (TestResult.Error);

			for (int i = 0; i < unknownCount; i++)
				results.Add (TestResult.Unknown);

			for (int i = 0; i < notEvaluatedCount; i++)
				results.Add (TestResult.NotEvaluated);

			for (int i = 0; i < notApplicableCount; i++)
				results.Add (TestResult.NotApplicable);

			return results;
		}

		[Test]
		public void TestAnalyzerByCheckStateMachine ()
		{
			// all states
			ArrayList allTrueResults = CreateResults (5, 0, 0, 0, 0, 0);
			TestResult res = DataAnalyzer.AnalyzeResultsByCheck (allTrueResults, "all");
			Assert.AreEqual (res, TestResult.True);

			Assert.AreEqual (res,  TestResult.True);

			ArrayList allFalseResults = CreateResults (0, 1, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (allFalseResults, "all");
			Assert.AreEqual (res,  TestResult.False);

			ArrayList allErrorResults = CreateResults (0, 0, 1, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (allErrorResults, "all");
			Assert.AreEqual (res,  TestResult.Error);

			ArrayList allUnknownResults = CreateResults (1, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (allUnknownResults, "all");
			Assert.AreEqual (res,  TestResult.Unknown);

			ArrayList allNEResults = CreateResults (1, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (allNEResults, "all");
			Assert.AreEqual (res,  TestResult.NotEvaluated);

			ArrayList allNAResults = CreateResults (1, 0, 0, 0, 0, 1);
			res = DataAnalyzer.AnalyzeResultsByCheck (allNAResults, "all");
			Assert.AreEqual (res,  TestResult.NotApplicable);

			// at least one
			ArrayList aloTrueResults = CreateResults (1, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (aloTrueResults, "at least one");
			Assert.AreEqual (res,  TestResult.True);

			ArrayList aloFalseResults = CreateResults (0, 1, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (aloFalseResults, "at least one");
			Assert.AreEqual (res,  TestResult.False);

			ArrayList aloErrorResults = CreateResults (0, 0, 1, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (aloErrorResults, "at least one");
			Assert.AreEqual (res,  TestResult.Error);

			ArrayList aloUnknownResults = CreateResults (0, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (aloUnknownResults, "at least one");
			Assert.AreEqual (res,  TestResult.Unknown);

			ArrayList aloNEResults = CreateResults (0, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (aloNEResults, "at least one");
			Assert.AreEqual (res,  TestResult.NotEvaluated);

			ArrayList aloNAResults = CreateResults (0, 0, 0, 0, 0, 2);
			res = DataAnalyzer.AnalyzeResultsByCheck (aloNAResults, "at least one");
			Assert.AreEqual (res,  TestResult.NotApplicable);

			// only one
			ArrayList ooTrueResults = CreateResults (1, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooTrueResults, "only one");
			Assert.AreEqual (res,  TestResult.True);

			ArrayList ooFalseResults = CreateResults (3, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooFalseResults, "only one");
			Assert.AreEqual (res,  TestResult.False);

			ArrayList ooFalseResults2 = CreateResults (0, 1, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooFalseResults2, "only one");
			Assert.AreEqual (res,  TestResult.False);

			ArrayList ooErrorResults = CreateResults (0, 0, 1, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooErrorResults, "only one");
			Assert.AreEqual (res,  TestResult.Error);

			ArrayList ooUnknownResults = CreateResults (0, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooUnknownResults, "only one");
			Assert.AreEqual (res,  TestResult.Unknown);

			ArrayList ooNEResults = CreateResults (0, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooNEResults, "only one");
			Assert.AreEqual (res,  TestResult.NotEvaluated);

			ArrayList ooNAResults = CreateResults (0, 0, 0, 0, 0, 1);
			res = DataAnalyzer.AnalyzeResultsByCheck (ooNAResults, "only one");
			Assert.AreEqual (res,  TestResult.NotApplicable);

			// none
			ArrayList noneTrueResults = CreateResults (0, 1, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (noneTrueResults, "none");
			Assert.AreEqual (res,  TestResult.True);

			ArrayList noneFalseResults = CreateResults (1, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (noneFalseResults, "none");
			Assert.AreEqual (res,  TestResult.False);

			ArrayList noneErrorResults = CreateResults (0, 0, 1, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (noneErrorResults, "none");
			Assert.AreEqual (res,  TestResult.Error);

			ArrayList noneUnknownResults = CreateResults (0, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (noneUnknownResults, "none");
			Assert.AreEqual (res,  TestResult.Unknown);

			ArrayList noneNEResults = CreateResults (0, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResultsByCheck (noneNEResults, "none");
			Assert.AreEqual (res,  TestResult.NotEvaluated);

			ArrayList noneNAResults = CreateResults (0, 0, 0, 0, 0, 1);
			res = DataAnalyzer.AnalyzeResultsByCheck (noneNAResults, "none");
			Assert.AreEqual (res,  TestResult.NotApplicable);
		}
	}
}
