//
// AnalyzeByOperatorTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Collections;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Unix;

namespace SussenTests.Oval.Unix
{
	[TestFixture]
	public class AnalyzeByOperatorTest
	{
		[SetUp]
		public void SetUp()
		{
		}

		[TearDown]
		public void TearDown ()
		{
		}

		public ArrayList CreateResults (int trueCount, int falseCount, int errorCount, int unknownCount, int notEvaluatedCount, int notApplicableCount)
		{
			ArrayList results = new ArrayList ();

			for (int i = 0; i < trueCount; i++)
				results.Add (TestResult.True);

			for (int i = 0; i < falseCount; i++)
				results.Add (TestResult.False);

			for (int i = 0; i < errorCount; i++)
				results.Add (TestResult.Error);

			for (int i = 0; i < unknownCount; i++)
				results.Add (TestResult.Unknown);

			for (int i = 0; i < notEvaluatedCount; i++)
				results.Add (TestResult.NotEvaluated);

			for (int i = 0; i < notApplicableCount; i++)
				results.Add (TestResult.NotApplicable);

			return results;
		}

		[Test]
		public void TestAnalyzerByOperatorStateMachine ()
		{
			// AND states
			ArrayList andTrueResults = CreateResults (2, 0, 0, 0, 0, 0);
			TestResult res = DataAnalyzer.AnalyzeResults (andTrueResults, "AND");
			Assert.AreEqual (res,  TestResult.True);

			ArrayList andFalseResults = CreateResults (2, 1, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (andFalseResults, "AND");
			Assert.AreEqual (res,  TestResult.False);

			ArrayList andErrorResults = CreateResults (3, 0, 2, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (andErrorResults, "AND");
			Assert.AreEqual (res,  TestResult.Error);

			ArrayList andUnknownResults = CreateResults (1, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResults (andUnknownResults, "AND");
			Assert.AreEqual (res,  TestResult.Unknown);

			ArrayList andNEResults = CreateResults (1, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResults (andNEResults, "AND");
			Assert.AreEqual (res,  TestResult.NotEvaluated);

			ArrayList andNAResults = CreateResults (1, 0, 0, 0, 0, 1);
			res = DataAnalyzer.AnalyzeResults (andNAResults, "AND");
			Assert.AreEqual (res,  TestResult.NotApplicable);

			// OR states
			ArrayList orTrueResults = CreateResults (2, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (orTrueResults, "OR");
			Assert.AreEqual (res,  TestResult.True);

			ArrayList orFalseResults = CreateResults (0, 1, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (orFalseResults, "OR");
			Assert.AreEqual (res,  TestResult.False);

			ArrayList orErrorResults = CreateResults (0, 0, 2, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (orErrorResults, "OR");
			Assert.AreEqual (res,  TestResult.Error);

			ArrayList orUnknownResults = CreateResults (0, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResults (orUnknownResults, "OR");
			Assert.AreEqual (res,  TestResult.Unknown);

			ArrayList orNEResults = CreateResults (0, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResults (orNEResults, "OR");
			Assert.AreEqual (res,  TestResult.NotEvaluated);

			ArrayList orNAResults = CreateResults (0, 0, 0, 0, 0, 1);
			res = DataAnalyzer.AnalyzeResults (orNAResults, "OR");
			Assert.AreEqual (res,  TestResult.NotApplicable);

			// XOR states
			ArrayList xorTrueResults = CreateResults (3, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (xorTrueResults, "XOR");
			Assert.AreEqual (res,  TestResult.True);

			ArrayList xorFalseResults = CreateResults (2, 0, 0, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (xorFalseResults, "XOR");
			Assert.AreEqual (res,  TestResult.False);

			ArrayList xorErrorResults = CreateResults (0, 0, 1, 0, 0, 0);
			res = DataAnalyzer.AnalyzeResults (xorErrorResults, "XOR");
			Assert.AreEqual (res,  TestResult.Error);

			ArrayList xorUnknownResults = CreateResults (0, 0, 0, 1, 0, 0);
			res = DataAnalyzer.AnalyzeResults (xorUnknownResults, "XOR");
			Assert.AreEqual (res,  TestResult.Unknown);

			ArrayList xorNEResults = CreateResults (0, 0, 0, 0, 1, 0);
			res = DataAnalyzer.AnalyzeResults (xorNEResults, "XOR");
			Assert.AreEqual (res,  TestResult.NotEvaluated);

			ArrayList xorNAResults = CreateResults (0, 0, 0, 0, 0, 1);
			res = DataAnalyzer.AnalyzeResults (xorNAResults, "XOR");
			Assert.AreEqual (res,  TestResult.NotApplicable);
		}
	}
}
