//
// TortureTests.cs - NUnit Test Cases for Sussen.Oval
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Collections;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Apache;
using Sussen.Oval.Independent;
using Sussen.Oval.Unix;


namespace SussenTests.Oval
{
	[TestFixture]
	public class TortureTests
	{
		Config config;

		[SetUp]
		public void SetUp()
		{
			config = new Config ("linux");
		}

		[TearDown]
		public void TearDown ()
		{
			config = null;
		}

		[Test]
		[ExpectedException(typeof(XmlException))]
		public void TestInvalidXml ()
		{
			config.DefinitionsFile = "TortureTests.cs";
			new Interpreter (config);
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestNullConfig ()
		{
			new Interpreter (null);
		}

		[Test]
		[ExpectedException(typeof(ArgumentOutOfRangeException))]
		public void TestConfigInvalidFamily ()
		{
			config = new Config ("shackleford");
			new Interpreter (config);
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestConfigNoOvalDocFilename ()
		{
			new Interpreter (config);
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestConfigNoAccessSystemCharacteristicsFile ()
		{
			config.SystemCharacteristicsFile = "/root/bad-file-name.xml";
			config.DefinitionsFile = "oval.v5.test.definitions.xml";
			Interpreter i = new Interpreter (config);
			i.CollectData ();
			i.AnalyzeData ();
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestConfigDoesNotExistSystemCharacteristicsFile ()
		{
			config.SystemCharacteristicsFile = "/does/not/exist/bad-file-name.xml";
			config.DefinitionsFile = "oval.v5.test.definitions.xml";
			Interpreter i = new Interpreter (config);
			i.CollectData ();
			i.AnalyzeData ();
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestConfigBadMD5DefinitionsFile ()
		{
			config.DefinitionsFile = "oval.v5.test.definitions.xml";
			config.DefinitionsFileHash = "notlikely";
			config.VerifyMD5 = true;

			Interpreter i = new Interpreter (config);
			i.CollectData ();
			i.AnalyzeData ();
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestConfigNoAccessResultsFile ()
		{
			config.ResultsFile = "/root/bad-file-name.xml";
			config.DefinitionsFile = "oval.v5.test.definitions.xml";
			Interpreter i = new Interpreter (config);
			i.CollectData ();
			i.AnalyzeData ();
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestConfigDoesNotExistResultsFile ()
		{
			config.ResultsFile = "/does/not/exist/bad-file-name.xml";
			config.DefinitionsFile = "oval.v5.test.definitions.xml";
			Interpreter i = new Interpreter (config);
			i.CollectData ();
			i.AnalyzeData ();
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void TestDataCollectorNullConfig ()
		{
			new DataCollector (null);
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void TestDataCollectorEmptyConfig ()
		{			 
			OvalDocuments docs = null;
			DataCollector dc = new DataCollector (docs);
			dc.Run ();
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void TestDataAnalyzerNullConfig ()
		{
			new DataAnalyzer (null);
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void TestDataAnalyzerEmptyConfig ()
		{
			OvalDocuments docs = null;
			DataAnalyzer da = new DataAnalyzer (docs);
			da.Run ();
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void TestDefinitionAnalyzerNullArguments ()
		{
			new DefinitionAnalyzer (null, null);
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void TestTestAnalyzerNullArguments ()
		{
			TestAnalyzer ta = new TestAnalyzer ();
			ta.Run (null, null);
		}
	}
}
