//
// FileStateTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Unix;

namespace SussenTests.Oval.Unix
{
	[TestFixture]
	public class FileStateTest
	{
		Config config;
		DataCollector dc;
		OvalDocuments docs;

		[SetUp]
		public void SetUp()
		{
			config = new Config ("linux");
			config.DefinitionsFile = "oval.v5.test.definitions.xml";

			docs = new OvalDocuments ();
		}

		[TearDown]
		public void TearDown ()
		{
			config = null;
			docs = null;
		}

		void UnixCreateTest (TestElement testElement, ObjectElement objectElement)
		{
			if (testElement.Name == "file_test")
				dc.RunTest (new FileTest(), testElement, objectElement);
		}

		[Test]
		public void TestFileState ()
		{
			docs.Definitions.Load (config.DefinitionsFile);
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			docs.SystemCharacteristics.Create ();
			docs.SystemCharacteristics.WriteSystemInfo (new UnixSystemInfo());

			dc = new DataCollector (docs);
			dc.CreateTest = new PlatformCreateTest (UnixCreateTest);
			dc.Run ();
			
			DataAnalyzer da = new DataAnalyzer (docs);
			da.Run ();

			XmlNodeList nl = docs.Results.GetElementsByName ("results");
			if (!(nl.Count > 0))
				Assert.Fail ("No results element found in results document");
				
			XmlElement results = (XmlElement) nl[0];
			nl = results.GetElementsByTagName ("definitions");
			if (!(nl.Count > 0))
				Assert.Fail ("No definitions element found in results document");

			XmlElement definitions = (XmlElement) nl[0];		
			foreach (XmlElement def in definitions.ChildNodes) {
				string id = def.GetAttribute ("definition_id");
				string res = def.GetAttribute ("result");

				if (id == "oval:org.mmgsecurity:def:7")
					if (res != "true")
						Assert.Fail ("File state test failed");
			}
		}
	}
}
