<?xml version="1.0" encoding="UTF-8"?>
<!--
	AUTHOR:Matt Burton, The Mitre Corporation 
	DATE: 02May05 
	
	Modified by Loren Bandiera, MMG Security
   	   * Updating for v5 results
	DATE: 10May06

	The results_to_html stylesheet takes the results of a system's OVAL scan and displays it in a table with pertinent information
	to the scan such as all OVAL ID, CVE ID, Descriptions, OVAL version & status of the vulnerabilities checked.  
	The table is color coded, with a key at the top of the page, to group the vulnerabilities into three separate 
	categories of (1) Vulnerable, (2) Not Vulnerable, and (3) Unknown.
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:oval_results="http://oval.mitre.org/XMLSchema/oval-results-5"
	xmlns:oval_sc="http://oval.mitre.org/XMLSchema/oval-system-characteristics-5"
	xmlns:oval="http://oval.mitre.org/XMLSchema/oval-common-5"
	xmlns:oval_def="http://oval.mitre.org/XMLSchema/oval-definitions-5"
	xmlns:apache_def="http://oval.mitre.org/XMLSchema/oval-definitions-5#apache"
	xmlns:ind_def="http://oval.mitre.org/XMLSchema/oval-definitions-5#independent"
	xmlns:windows_def="http://oval.mitre.org/XMLSchema/oval-definitions-5#windows"
	xmlns:unix_def="http://oval.mitre.org/XMLSchema/oval-definitions-5#unix"
	xmlns:linux_def="http://oval.mitre.org/XMLSchema/oval-definitions-5#linux">
	<xsl:output method="html"/>
	
	<!-- Style declarations for pretty formatting -->
	<xsl:template name="oval_results_style">
		<style type="text/css">
			<xsl:comment>
TD.title {BACKGROUND-COLOR: #000000; COLOR: #ffc; FONT-WEIGHT: bold; TEXT-ALIGN: left} TD.label {BACKGROUND-COLOR: #99cc99}
TR.vulnerable {BACKGROUND-COLOR: #FF4040} TR.notvulnerable {BACKGROUND-COLOR: #8DB6CD} TR.unknown {BACKGROUND-COLOR: #FFFF99}
TD.vulnerable {BACKGROUND-COLOR: #FF4040; text-align: center} TD.notvulnerable {BACKGROUND-COLOR: #8DB6CD; text-align: center} TD.unknown {BACKGROUND-COLOR: #FFFF99; text-align: center}
TD.label2 {FONT-WEIGHT: bold}
			</xsl:comment>
		</style>
	</xsl:template>
	
	<!-- Gets the top level node -->
	<xsl:template match="oval_results:oval_results">
		<HTML>
			<HEAD>
				<TITLE> OVAL Results </TITLE>
				<!-- Get stylesheet -->
				<xsl:call-template name="oval_results_style"/>
			</HEAD>
			<BODY>
				<!-- Get pertinent information in this order: system_info, generators, definitions --> 
				<xsl:apply-templates select="oval_results:results/oval_results:system/oval_sc:oval_system_characteristics/oval_sc:system_info"/>
				<xsl:apply-templates select="oval_results:generator"/>
				<xsl:apply-templates select="oval_def:oval_definitions"/>
			</BODY>
		</HTML>
	</xsl:template>
	

	<!-- Get the system_info and put it into a table. -->
	<xsl:template match="oval_results:results/oval_results:system/oval_sc:oval_system_characteristics/oval_sc:system_info">
		<table border="1" cellspacing="0" cellpaddign="2" width="100%" bgcolor="#cccccc">
			<tr><td class="title" colspan="2">System Information</td></tr>
			<tr>
				<td class="label2" width="20%">Host Name</td>
				<td width="80%"><xsl:value-of select="oval_sc:primary_host_name"/>&#160;</td> <!-- $#160; is used to keep empty cells in the table clean -->
			</tr>
			<tr>
				<td class="label2" width="20%">Operating System</td>
				<td width="80%"><xsl:value-of select="oval_sc:os_name"/>&#160;</td>
			</tr>
			<tr>
				<td class="label2" width="20%">Operating System Version</td>
				<td width="80%"><xsl:value-of select="oval_sc:os_version"/>&#160;</td>
			</tr>
			<tr>
				<td class="label2" width="20%">Architecture</td>
				<td  width="80%"><xsl:value-of select="oval_sc:architecture"/>&#160;</td></tr>
			<tr><td class="label2" colspan="2">Interfaces</td></tr>
			<xsl:apply-templates select="oval_sc:interfaces"/>	
		</table>
	</xsl:template>

	<!-- Get this interface and put it into the table, using templates to loop through all possible interfaces -->
	<xsl:template match="oval_sc:interfaces">
		<xsl:for-each select="oval_sc:interface">
			<tr>
				<td class="label2" width="20%">Interface Name</td>
				<td  width="80%"><xsl:value-of select="oval_sc:interface_name"/>&#160;</td>
			</tr>
			<tr>
				<td class="label2" width="20%">IP Address</td>
				<td width="80%"><xsl:value-of select="oval_sc:ip_address"></xsl:value-of>&#160;</td>
			</tr>
			<tr>
				<td class="label2" width="20%">MAC Address</td>
				<td width="80%"><xsl:value-of select="oval_sc:mac_address"></xsl:value-of>&#160;</td>
			</tr>
		</xsl:for-each>
	</xsl:template>
	
	<!-- Crate a table for Generators' and call its children. -->
	<xsl:template match="oval_results:generator">
		<table border="1" cellspacing="0" cellpaddign="2" width="100%" bgcolor="#cccccc">
			<tr><td class="title" colspan="5">Schema Information</td></tr>
			<tr>
				<td class="label">Schema Name</td>
				<td class="label">Schema Version</td>
				<td class="label">Product Name</td>
				<td class="label">Product Version</td>
				<td class="label">Timestamp</td>
			</tr>
			<xsl:call-template name="generator_info"/>
<!--			<xsl:apply-templates/> -->
		</table>
	</xsl:template>
	
	<!-- Templates for each of Generators' children that in turn call a generic table. -->
	<xsl:template match="oval_results:oval">
		<tr>
			<td width="20%">Oval Schema</td>
			<xsl:call-template name="generator_info"/>
		</tr>
	</xsl:template>

	<xsl:template match="oval_def:oval_definitions/oval_def:generator" />

<!--
	<xsl:template match="oval_results:oval_definitions/oval_def:generator">
		<tr>
			<td width="20%">System Characteristics Schema</td>
			<xsl:call-template name="generator_info"/>
		</tr>
	</xsl:template>

	<xsl:template match="oval_results:results">
		<tr>
			<td width="20%">Results Schema</td>
			<xsl:call-template name="generator_info"/>
		</tr>
	</xsl:template>
-->

	<!-- Each child of Generators (oval, system_characteristics, results) is the same, this template gets their children. -->
	<xsl:template name="generator_info">
		<xsl:variable name="MessyNumber" select="string(oval:timestamp)"/>
		<td><xsl:value-of select="oval:schema_version"/>&#160;</td>
		<td><xsl:value-of select="oval:product_name"/>&#160;</td>
		<td><xsl:value-of select="oval:product_version"/>&#160;</td>
		<td>
			<!--Create variable "MessyNumber" to make time stamp a string and then print it out in a readable version -->
			<xsl:value-of select="substring($MessyNumber, 1, 4)"/>- <!-- year -->
			<xsl:value-of select="substring($MessyNumber, 6, 2)"/>- <!-- month -->
			<xsl:value-of select="substring($MessyNumber, 9, 2)"/> <!-- day -->
		</td>
		<td>		
			<xsl:value-of select="substring($MessyNumber, 12, 2)"/> <xsl:text>:</xsl:text> <!-- hour -->
			<xsl:value-of select="substring($MessyNumber, 15, 2)"/> <xsl:text>:</xsl:text> <!-- minute -->
			<xsl:value-of select="substring($MessyNumber, 18, 2)"/>&#160; <!-- second -->
		</td>
	</xsl:template>
	
	
	
	<xsl:template match="oval_def:oval_definitions">
		<table border="1" cellspacing="0" cellpaddign="2" width="100%">
			<tr><td class="title" colspan="5">Oval Test Results</td></tr>
			<!-- ^^^^^^^^^^^^^^^^^ Table Color Key table ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ -->
			<tr>
					<td colspan="5">	
						<table cellspacing="0" border="0" width="100%">
							<td class="vulnerable" width="33%">Vulnerable</td>
							<td class="notvulnerable" width="33%">Not Vulnerable</td>
							<td class="unknown" width="33%">Unknown</td>
						</table>  
					</td>
			</tr>
<!-- ^^^^^^^^^^^^^^^^^ End of Table Color Key table ^^^^^^^^^^^^^^^^^^^^^^^^^ -->
			<tr>
				<td class="label" align="center">OVAL ID</td>
				<td class="label" align="center">CVE ID</td>
				<td class="label" align="center">Description</td>
				<td class="label" align="center">Status</td>
				<td class="label" align="center">Version</td>
			</tr>
			<xsl:apply-templates>
				<!-- sort vulnerabilities into groups of vulnerable, not vulnerable and unknown -->
				<!--This has been disabled until I figure out how to sort both software & configuration-->
				<xsl:sort select="oval_results:criteria/@result" data-type="number" order="descending"/>
			</xsl:apply-templates>
		</table>
	</xsl:template>
	
	<xsl:template match="oval_def:definition">
	 
		<!-- Start of new row -->
		<tr>
			<xsl:choose>
				<xsl:when test="oval_results:criteria/@result">
					<!-- set the background color for the row according to vulnerable/not vulnerable/unknown status-->
					<xsl:choose>
						<xsl:when test="oval_results:criteria/@result='true'"> <!-- if vulnerable, sets this color for row -->
							<xsl:attribute name="class">vulnerable</xsl:attribute>
						</xsl:when>
						<xsl:when test="./oval_results:criteria/@result='false'"> <!-- if not vulnerable, sets this color for row -->
							<xsl:attribute name="class">notvulnerable</xsl:attribute>
						</xsl:when>				
						<xsl:otherwise>
							<xsl:attribute name="class">unknown</xsl:attribute> <!-- if error occurs, selects default 'unknown' color for row-->
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<!-- set the background color for the row according to vulnerable/not vulnerable/unknown status-->
						<xsl:choose>
							<xsl:when test="oval_results:criteria/@result='true'"> <!-- if vulnerable, sets this color for row -->
								<xsl:attribute name="class">vulnerable</xsl:attribute>
							</xsl:when>
							<xsl:when test="./oval_results:criteria/@result='false'"> <!-- if not vulnerable, sets this color for row -->
								<xsl:attribute name="class">notvulnerable</xsl:attribute>
							</xsl:when>					
							<xsl:otherwise>
								<xsl:attribute name="class">unknown</xsl:attribute> <!-- if error occurs, selects default 'unknown' color for row-->
							</xsl:otherwise>
						</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
		
			<td width="100" align="center"> 
				<xsl:text disable-output-escaping='yes'>&lt;a </xsl:text>
					<xsl:text>href="http://oval.mitre.org/oval/definitions/data/oval</xsl:text> <!-- OVAL url -->
					<xsl:value-of select='substring-after(@id,"def:")'/>
					<xsl:text>.html" target="_blank"></xsl:text>
					<xsl:text disable-output-escaping="yes">&gt;&#10;</xsl:text>
					<xsl:value-of select="@id"/> <!-- writes OVAL ID to first column -->
				<xsl:text disable-output-escaping="yes">&lt;/a&gt;&#10;</xsl:text>
			</td>
			<td width="150" align="center">
				<!-- creates hyperlink to appropriate CVE site -->
				<xsl:text disable-output-escaping='yes'>&lt;a </xsl:text>
						<xsl:text>href="</xsl:text>
							<xsl:value-of select="oval_def:metadata/oval_def:reference/@ref_url"/> <!-- template call to get CAN/CVE -->
						<xsl:text>"  target="_blank"></xsl:text>
					<xsl:text disable-output-escaping="yes">&gt;&#10;</xsl:text>
						<xsl:value-of select="oval_def:metadata/oval_def:reference/@ref_id"/>		<!-- template call to get CAN/CVE & print -->
				<xsl:text disable-output-escaping="yes">&lt;/a&gt;&#10;</xsl:text>
			</td>
			<td width="700" align="left"> <!-- sets width and alignment for vulnerability description column -->
			
				<xsl:value-of select="oval_def:metadata/oval_def:description"/> <!-- writes vulnerability description to 3rd column -->
			</td>
			
			<td width="100" align="center"> <!-- sets width and alignment for vulnerability status column -->
			
				<xsl:value-of select="oval_def:metadata/oval_def:oval_repository/oval_def:status"/> <!-- writes vulnerability status to 4th column -->
			</td>
			
			<td width="100" align="center"> <!-- sets width and alignment for vulnerability version column -->
			
				<xsl:value-of select="@version"/> <!-- writes vulnerability version to 5th column -->
			</td>
		</tr>
			
	</xsl:template>

	<xsl:template match="oval_def:variables" />

	<!-- Apache -->
	<xsl:template match="apache_def:version_test" />
	<xsl:template match="apache_def:version_object" />
	<xsl:template match="apache_def:version_state" />

	<!-- Independent -->
	<xsl:template match="ind_def:family_test" />
	<xsl:template match="ind_def:family_object" />
	<xsl:template match="ind_def:family_state" />

	<xsl:template match="ind_def:filemd5_test" />
	<xsl:template match="ind_def:filemd5_object" />
	<xsl:template match="ind_def:filemd5_state" />

	<xsl:template match="ind_def:environmentvariable_test" />
	<xsl:template match="ind_def:environmentvariable_object" />
	<xsl:template match="ind_def:environmentvariable_state" />

	<xsl:template match="ind_def:textfilecontent_test" />
	<xsl:template match="ind_def:textfilecontent_object" />
	<xsl:template match="ind_def:textfilecontent_state" />

	<xsl:template match="ind_def:xmlfilecontent_test" />
	<xsl:template match="ind_def:xmlfilecontent_object" />
	<xsl:template match="ind_def:xmlfilecontent_state" />

	<xsl:template match="ind_def:unknown_test" />
	<xsl:template match="ind_def:unknown_object" />
	<xsl:template match="ind_def:unknown_state" />

	<xsl:template match="ind_def:variable_test" />
	<xsl:template match="ind_def:variable_object" />
	<xsl:template match="ind_def:variable_state" />

	<!-- Linux -->
	<xsl:template match="linux_def:dpkginfo_test" />
	<xsl:template match="linux_def:dpkginfo_object" />
	<xsl:template match="linux_def:dpkginfo_state" />

	<xsl:template match="linux_def:rpminfo_test" />
	<xsl:template match="linux_def:rpminfo_object" />
	<xsl:template match="linux_def:rpminfo_state" />

	<xsl:template match="linux_def:inetlisteningservers_test" />
	<xsl:template match="linux_def:inetlisteningservers_object" />
	<xsl:template match="linux_def:inetlisteningservers_state" />

	<!-- Unix -->
	<xsl:template match="unix_def:file_test" />
	<xsl:template match="unix_def:file_object" />
	<xsl:template match="unix_def:file_state" />

	<xsl:template match="unix_def:inetd_test" />
	<xsl:template match="unix_def:inetd_object" />
	<xsl:template match="unix_def:inetd_state" />

	<xsl:template match="unix_def:interface_test" />
	<xsl:template match="unix_def:interface_object" />
	<xsl:template match="unix_def:interface_state" />

	<xsl:template match="unix_def:password_test" />
	<xsl:template match="unix_def:password_object" />
	<xsl:template match="unix_def:password_state" />

	<xsl:template match="unix_def:process_test" />
	<xsl:template match="unix_def:process_object" />
	<xsl:template match="unix_def:process_state" />

	<xsl:template match="unix_def:runlevel_test" />
	<xsl:template match="unix_def:runlevel_object" />
	<xsl:template match="unix_def:runlevel_state" />

	<xsl:template match="unix_def:sccs_test" />
	<xsl:template match="unix_def:sccs_object" />
	<xsl:template match="unix_def:sccs_state" />

	<xsl:template match="unix_def:shadow_test" />
	<xsl:template match="unix_def:shadow_object" />
	<xsl:template match="unix_def:shadow_state" />

	<xsl:template match="unix_def:uname_test" />
	<xsl:template match="unix_def:uname_object" />
	<xsl:template match="unix_def:uname_state" />

	<!-- Windows -->
	<xsl:template match="windows_def:accesstoken_test" />
	<xsl:template match="windows_def:accesstoken_object" />
	<xsl:template match="windows_def:accesstoken_state" />

	<xsl:template match="windows_def:activedirectory_test" />
	<xsl:template match="windows_def:activedirectory_object" />
	<xsl:template match="windows_def:activedirectory_state" />

	<xsl:template match="windows_def:auditeventpolicy_test" />
	<xsl:template match="windows_def:auditeventpolicy_object" />
	<xsl:template match="windows_def:auditeventpolicy_state" />

	<xsl:template match="windows_def:file_test" />
	<xsl:template match="windows_def:file_object" />
	<xsl:template match="windows_def:file_state" />

	<xsl:template match="windows_def:fileauditpermissions_test" />
	<xsl:template match="windows_def:fileauditpermissions_object" />
	<xsl:template match="windows_def:fileauditpermissions_state" />

	<xsl:template match="windows_def:fileeffectiverights_test" />
	<xsl:template match="windows_def:fileeffectiverights_object" />
	<xsl:template match="windows_def:fileeffectiverights_state" />

	<xsl:template match="windows_def:group_test" />
	<xsl:template match="windows_def:group_object" />
	<xsl:template match="windows_def:group_state" />

	<xsl:template match="windows_def:interface_test" />
	<xsl:template match="windows_def:interface_object" />
	<xsl:template match="windows_def:interface_state" />

	<xsl:template match="windows_def:lockoutpolicy_test" />
	<xsl:template match="windows_def:lockoutpolicy_object" />
	<xsl:template match="windows_def:lockoutpolicy_state" />

	<xsl:template match="windows_def:metabase_test" />
	<xsl:template match="windows_def:metabase_object" />
	<xsl:template match="windows_def:metabase_state" />

	<xsl:template match="windows_def:passwordpolicy_test" />
	<xsl:template match="windows_def:passwordpolicy_object" />
	<xsl:template match="windows_def:passwordpolicy_state" />

	<xsl:template match="windows_def:port_test" />
	<xsl:template match="windows_def:port_object" />
	<xsl:template match="windows_def:port_state" />

	<xsl:template match="windows_def:process_test" />
	<xsl:template match="windows_def:process_object" />
	<xsl:template match="windows_def:process_state" />

	<xsl:template match="windows_def:registry_test" />
	<xsl:template match="windows_def:registry_object" />
	<xsl:template match="windows_def:registry_state" />

	<xsl:template match="windows_def:regkeyauditedpermissions_test" />
	<xsl:template match="windows_def:regkeyauditedpermissions_object" />
	<xsl:template match="windows_def:regkeyauditedpermissions_state" />

	<xsl:template match="windows_def:regkeyeffectiverights_test" />
	<xsl:template match="windows_def:regkeyeffectiverights_object" />
	<xsl:template match="windows_def:regkeyeffectiverights_state" />

	<xsl:template match="windows_def:sid_test" />
	<xsl:template match="windows_def:sid_object" />
	<xsl:template match="windows_def:sid_state" />

	<xsl:template match="windows_def:user_test" />
	<xsl:template match="windows_def:user_object" />
	<xsl:template match="windows_def:user_state" />

	<xsl:template match="windows_def:volume_test" />
	<xsl:template match="windows_def:volume_object" />
	<xsl:template match="windows_def:volume_state" />

	<xsl:template match="windows_def:wmi_test" />
	<xsl:template match="windows_def:wmi_object" />
	<xsl:template match="windows_def:wmi_state" />

</xsl:stylesheet>
